/*
 *  vbfamprojectmanager.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <am/vbfamprojectmanager.h>
#include <glib/gstdio.h>
#include <gee/list.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <gee/arraylist.h>
#include <gee/collection.h>
#include <vbfmodule.h>
#include <vbfpackage.h>
#include <vbfconfignode.h>
#include <vbfunresolvedconfignode.h>
#include <vbfstringliteral.h>
#include <vbfgroup.h>
#include <vbfvariable.h>
#include <vbfconfignodelist.h>
#include <vbfconfignodepair.h>
#include <vbftarget.h>
#include <vbfsource.h>
#include <vbffile.h>




static char* string_replace (const char* self, const char* old, const char* replacement);
static char* string_substring (const char* self, glong offset, glong len);
static glong string_get_length (const char* self);
enum  {
	VBF_AM_PROJECT_MANAGER_DUMMY_PROPERTY
};
static gboolean vbf_am_project_manager_real_probe (VbfIProjectManager* base, const char* project_file);
static VbfProject* vbf_am_project_manager_real_open (VbfIProjectManager* base, const char* project_file);
static void vbf_am_project_manager_real_refresh (VbfIProjectManager* base, VbfProject* project);
static void vbf_am_project_manager_add_vala_source (VbfAmProjectManager* self, VbfGroup* group, VbfTarget* target, VbfConfigNode* source);
static void vbf_am_project_manager_add_vala_sources (VbfAmProjectManager* self, VbfGroup* group, VbfTarget* target);
static void vbf_am_project_manager_add_targets (VbfAmProjectManager* self, VbfGroup* group, VbfConfigNode* node, VbfTargetTypes type);
static void vbf_am_project_manager_add_data_files (VbfAmProjectManager* self, VbfGroup* group, VbfTarget* target, VbfConfigNode* file);
static void vbf_am_project_manager_parse_targets (VbfAmProjectManager* self, VbfGroup* group);
static void vbf_am_project_manager_parse_variables (VbfAmProjectManager* self, VbfProject* project, const char* buffer, VbfGroup* group, GError** error);
static VbfVariable* vbf_am_project_manager_resolve_variable (VbfAmProjectManager* self, const char* variable_name, GeeList* variables);
static void vbf_am_project_manager_resolve_package_variables (VbfAmProjectManager* self, VbfProject* project);
static void vbf_am_project_manager_resolve_variables (VbfAmProjectManager* self, VbfProject* project, GeeList* variables);
static char* vbf_am_project_manager_process_include_directives (VbfAmProjectManager* self, const char* makefile, const char* buffer, GError** error);
static void vbf_am_project_manager_parse_makefile (VbfAmProjectManager* self, VbfProject* project, const char* project_file, const char* makefile, GError** error);
static void vbf_am_project_manager_parse_group_extended_info (VbfAmProjectManager* self, VbfGroup* group, const char* buffer);
static char* vbf_am_project_manager_normalize_string (VbfAmProjectManager* self, const char* data);
static gpointer vbf_am_project_manager_parent_class = NULL;
static VbfIProjectManagerIface* vbf_am_project_manager_vbf_iproject_manager_parent_iface = NULL;
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



static char* string_replace (const char* self, const char* old, const char* replacement) {
	GError * inner_error;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (old != NULL, NULL);
	g_return_val_if_fail (replacement != NULL, NULL);
	inner_error = NULL;
	{
		char* _tmp0;
		GRegex* _tmp1;
		GRegex* regex;
		char* _tmp2;
		char* _tmp3;
		_tmp0 = NULL;
		_tmp1 = NULL;
		regex = (_tmp1 = g_regex_new (_tmp0 = g_regex_escape_string (old, -1), 0, 0, &inner_error), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
		if (inner_error != NULL) {
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch2_g_regex_error;
			}
			goto __finally2;
		}
		_tmp2 = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &inner_error);
		if (inner_error != NULL) {
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch2_g_regex_error;
			}
			goto __finally2;
		}
		_tmp3 = NULL;
		return (_tmp3 = _tmp2, (regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL)), _tmp3);
	}
	goto __finally2;
	__catch2_g_regex_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			g_assert_not_reached ();
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally2:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return NULL;
	}
}


static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


/*
 * check if the given directory is a base source dir
 * of an autotool project.
 */
static gboolean vbf_am_project_manager_real_probe (VbfIProjectManager* base, const char* project_file) {
	VbfAmProjectManager * self;
	char* file;
	gboolean res;
	gboolean _tmp1;
	self = (VbfAmProjectManager*) base;
	g_return_val_if_fail (project_file != NULL, FALSE);
	file = g_build_filename (project_file, "configure.ac", NULL);
	res = FALSE;
	if (g_file_test (file, G_FILE_TEST_EXISTS)) {
		char* _tmp0;
		_tmp0 = NULL;
		file = (_tmp0 = g_build_filename (project_file, "Makefile.am", NULL), file = (g_free (file), NULL), _tmp0);
		if (g_file_test (file, G_FILE_TEST_EXISTS)) {
			res = TRUE;
		}
	}
	return (_tmp1 = res, file = (g_free (file), NULL), _tmp1);
}


static VbfProject* vbf_am_project_manager_real_open (VbfIProjectManager* base, const char* project_file) {
	VbfAmProjectManager * self;
	VbfProject* project;
	VbfIProjectManager* _tmp1;
	VbfIProjectManager* _tmp0;
	self = (VbfAmProjectManager*) base;
	g_return_val_if_fail (project_file != NULL, NULL);
	project = vbf_project_new (project_file);
	_tmp1 = NULL;
	_tmp0 = NULL;
	project->backend = (_tmp1 = (_tmp0 = (VbfIProjectManager*) self, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (project->backend == NULL) ? NULL : (project->backend = (g_object_unref (project->backend), NULL)), _tmp1);
	vbf_iproject_manager_refresh ((VbfIProjectManager*) self, project);
	if (project->name == NULL) {
		VbfProject* _tmp2;
		_tmp2 = NULL;
		return (_tmp2 = NULL, (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), _tmp2);
	} else {
		return project;
	}
	(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
}


static void vbf_am_project_manager_real_refresh (VbfIProjectManager* base, VbfProject* project) {
	VbfAmProjectManager * self;
	GError * inner_error;
	self = (VbfAmProjectManager*) base;
	g_return_if_fail (project != NULL);
	inner_error = NULL;
	{
		char* file;
		char* buffer;
		gulong length;
		char* _tmp2;
		gboolean _tmp1;
		char* _tmp0;
		gboolean _tmp3;
		GRegex* reg;
		GMatchInfo* match;
		char* name;
		char* version;
		char* url;
		GMatchInfo* _tmp6;
		gboolean _tmp5;
		GMatchInfo* _tmp4;
		char* _tmp28;
		char* _tmp29;
		char* _tmp30;
		char* _tmp32;
		const char* _tmp31;
		GRegex* _tmp33;
		GMatchInfo* _tmp36;
		gboolean _tmp35;
		GMatchInfo* _tmp34;
		GeeList* _tmp64;
		GRegex* _tmp65;
		GMatchInfo* _tmp68;
		gboolean _tmp67;
		GMatchInfo* _tmp66;
		gboolean res;
		file = g_build_filename (project->id, "configure.ac", NULL);
		buffer = NULL;
		length = 0UL;
		_tmp2 = NULL;
		_tmp0 = NULL;
		_tmp3 = (_tmp1 = g_file_get_contents (file, &_tmp0, &length, &inner_error), buffer = (_tmp2 = _tmp0, buffer = (g_free (buffer), NULL), _tmp2), _tmp1);
		if (inner_error != NULL) {
			file = (g_free (file), NULL);
			buffer = (g_free (buffer), NULL);
			goto __catch3_g_error;
			goto __finally3;
		}
		if (!_tmp3) {
			file = (g_free (file), NULL);
			buffer = (g_free (buffer), NULL);
			return;
		}
		reg = g_regex_new ("AC_INIT\\(([^\\,\\)]+)\\)", 0, 0, &inner_error);
		if (inner_error != NULL) {
			file = (g_free (file), NULL);
			buffer = (g_free (buffer), NULL);
			goto __catch3_g_error;
			goto __finally3;
		}
		match = NULL;
		name = NULL;
		version = NULL;
		url = NULL;
		_tmp6 = NULL;
		_tmp4 = NULL;
		if ((_tmp5 = g_regex_match (reg, buffer, G_REGEX_MATCH_NEWLINE_ANY, &_tmp4), match = (_tmp6 = _tmp4, (match == NULL) ? NULL : (match = (g_match_info_free (match), NULL)), _tmp6), _tmp5)) {
			char* _tmp7;
			_tmp7 = NULL;
			name = (_tmp7 = g_match_info_fetch (match, 1), name = (g_free (name), NULL), _tmp7);
		} else {
			GRegex* _tmp8;
			GMatchInfo* _tmp11;
			gboolean _tmp10;
			GMatchInfo* _tmp9;
			_tmp8 = NULL;
			reg = (_tmp8 = g_regex_new ("AC_INIT\\(([^\\,\\)]+),([^\\,\\)]+)\\)", 0, 0, &inner_error), (reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL)), _tmp8);
			if (inner_error != NULL) {
				file = (g_free (file), NULL);
				buffer = (g_free (buffer), NULL);
				(reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL));
				(match == NULL) ? NULL : (match = (g_match_info_free (match), NULL));
				name = (g_free (name), NULL);
				version = (g_free (version), NULL);
				url = (g_free (url), NULL);
				goto __catch3_g_error;
				goto __finally3;
			}
			_tmp11 = NULL;
			_tmp9 = NULL;
			if ((_tmp10 = g_regex_match (reg, buffer, G_REGEX_MATCH_NEWLINE_ANY, &_tmp9), match = (_tmp11 = _tmp9, (match == NULL) ? NULL : (match = (g_match_info_free (match), NULL)), _tmp11), _tmp10)) {
				char* _tmp12;
				char* _tmp13;
				_tmp12 = NULL;
				name = (_tmp12 = g_match_info_fetch (match, 1), name = (g_free (name), NULL), _tmp12);
				_tmp13 = NULL;
				version = (_tmp13 = g_match_info_fetch (match, 2), version = (g_free (version), NULL), _tmp13);
			} else {
				GRegex* _tmp14;
				GMatchInfo* _tmp17;
				gboolean _tmp16;
				GMatchInfo* _tmp15;
				_tmp14 = NULL;
				reg = (_tmp14 = g_regex_new ("AC_INIT\\(([^\\,\\)]+),([^\\,\\)]+),([^\\,\\)]+)\\)", 0, 0, &inner_error), (reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL)), _tmp14);
				if (inner_error != NULL) {
					file = (g_free (file), NULL);
					buffer = (g_free (buffer), NULL);
					(reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL));
					(match == NULL) ? NULL : (match = (g_match_info_free (match), NULL));
					name = (g_free (name), NULL);
					version = (g_free (version), NULL);
					url = (g_free (url), NULL);
					goto __catch3_g_error;
					goto __finally3;
				}
				_tmp17 = NULL;
				_tmp15 = NULL;
				if ((_tmp16 = g_regex_match (reg, buffer, G_REGEX_MATCH_NEWLINE_ANY, &_tmp15), match = (_tmp17 = _tmp15, (match == NULL) ? NULL : (match = (g_match_info_free (match), NULL)), _tmp17), _tmp16)) {
					char* _tmp18;
					char* _tmp19;
					char* _tmp20;
					_tmp18 = NULL;
					name = (_tmp18 = g_match_info_fetch (match, 1), name = (g_free (name), NULL), _tmp18);
					_tmp19 = NULL;
					version = (_tmp19 = g_match_info_fetch (match, 2), version = (g_free (version), NULL), _tmp19);
					_tmp20 = NULL;
					url = (_tmp20 = g_match_info_fetch (match, 3), url = (g_free (url), NULL), _tmp20);
				} else {
					GRegex* _tmp21;
					GMatchInfo* _tmp24;
					gboolean _tmp23;
					GMatchInfo* _tmp22;
					_tmp21 = NULL;
					reg = (_tmp21 = g_regex_new ("AC_INIT\\(([^\\,\\)]+),([^\\,\\)]+),([^\\,\\)]+),([^\\,\\)]+)\\)", 0, 0, &inner_error), (reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL)), _tmp21);
					if (inner_error != NULL) {
						file = (g_free (file), NULL);
						buffer = (g_free (buffer), NULL);
						(reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL));
						(match == NULL) ? NULL : (match = (g_match_info_free (match), NULL));
						name = (g_free (name), NULL);
						version = (g_free (version), NULL);
						url = (g_free (url), NULL);
						goto __catch3_g_error;
						goto __finally3;
					}
					_tmp24 = NULL;
					_tmp22 = NULL;
					if ((_tmp23 = g_regex_match (reg, buffer, G_REGEX_MATCH_NEWLINE_ANY, &_tmp22), match = (_tmp24 = _tmp22, (match == NULL) ? NULL : (match = (g_match_info_free (match), NULL)), _tmp24), _tmp23)) {
						char* _tmp25;
						char* _tmp26;
						char* _tmp27;
						_tmp25 = NULL;
						name = (_tmp25 = g_match_info_fetch (match, 1), name = (g_free (name), NULL), _tmp25);
						_tmp26 = NULL;
						version = (_tmp26 = g_match_info_fetch (match, 2), version = (g_free (version), NULL), _tmp26);
						_tmp27 = NULL;
						url = (_tmp27 = g_match_info_fetch (match, 3), url = (g_free (url), NULL), _tmp27);
					}
				}
			}
		}
		vbf_project_clear (project);
		_tmp28 = NULL;
		project->name = (_tmp28 = vbf_am_project_manager_normalize_string (self, name), project->name = (g_free (project->name), NULL), _tmp28);
		_tmp29 = NULL;
		project->version = (_tmp29 = vbf_am_project_manager_normalize_string (self, version), project->version = (g_free (project->version), NULL), _tmp29);
		_tmp30 = NULL;
		project->url = (_tmp30 = vbf_am_project_manager_normalize_string (self, url), project->url = (g_free (project->url), NULL), _tmp30);
		_tmp32 = NULL;
		_tmp31 = NULL;
		project->working_dir = (_tmp32 = (_tmp31 = project->id, (_tmp31 == NULL) ? NULL : g_strdup (_tmp31)), project->working_dir = (g_free (project->working_dir), NULL), _tmp32);
		/*Extract pkg-config packages*/
		_tmp33 = NULL;
		reg = (_tmp33 = g_regex_new ("PKG_CHECK_MODULES\\([\\s\\[]*([^\\,\\)\\]]+)[\\s\\]]*\\,(.+?)\\)", 0, 0, &inner_error), (reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL)), _tmp33);
		if (inner_error != NULL) {
			file = (g_free (file), NULL);
			buffer = (g_free (buffer), NULL);
			(reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL));
			(match == NULL) ? NULL : (match = (g_match_info_free (match), NULL));
			name = (g_free (name), NULL);
			version = (g_free (version), NULL);
			url = (g_free (url), NULL);
			goto __catch3_g_error;
			goto __finally3;
		}
		_tmp36 = NULL;
		_tmp34 = NULL;
		if ((_tmp35 = g_regex_match (reg, buffer, G_REGEX_MATCH_NEWLINE_CR, &_tmp34), match = (_tmp36 = _tmp34, (match == NULL) ? NULL : (match = (g_match_info_free (match), NULL)), _tmp36), _tmp35)) {
			while (g_match_info_matches (match)) {
				char* mod_name;
				char* pacs;
				char* line;
				GRegex* mod;
				GMatchInfo* pac;
				GMatchInfo* _tmp39;
				gboolean _tmp38;
				GMatchInfo* _tmp37;
				VbfModule* module;
				mod_name = g_match_info_fetch (match, 1);
				pacs = g_strdup ("");
				line = g_match_info_fetch (match, 2);
				mod = g_regex_new ("^[\\s\\]]*([^\\,\\]]+)[\\s\\]]*,([^\\,]+),([^\\,]+)$", 0, 0, &inner_error);
				if (inner_error != NULL) {
					mod_name = (g_free (mod_name), NULL);
					pacs = (g_free (pacs), NULL);
					line = (g_free (line), NULL);
					file = (g_free (file), NULL);
					buffer = (g_free (buffer), NULL);
					(reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL));
					(match == NULL) ? NULL : (match = (g_match_info_free (match), NULL));
					name = (g_free (name), NULL);
					version = (g_free (version), NULL);
					url = (g_free (url), NULL);
					goto __catch3_g_error;
					goto __finally3;
				}
				pac = NULL;
				_tmp39 = NULL;
				_tmp37 = NULL;
				if ((_tmp38 = g_regex_match (mod, line, G_REGEX_MATCH_NEWLINE_CR, &_tmp37), pac = (_tmp39 = _tmp37, (pac == NULL) ? NULL : (pac = (g_match_info_free (pac), NULL)), _tmp39), _tmp38)) {
					char* _tmp40;
					_tmp40 = NULL;
					pacs = (_tmp40 = g_match_info_fetch (pac, 1), pacs = (g_free (pacs), NULL), _tmp40);
				} else {
					GRegex* _tmp41;
					GMatchInfo* _tmp44;
					gboolean _tmp43;
					GMatchInfo* _tmp42;
					_tmp41 = NULL;
					mod = (_tmp41 = g_regex_new ("^[\\s\\]]*([^\\,\\]]+)[\\s\\]]*,([^\\,]+)$", 0, 0, &inner_error), (mod == NULL) ? NULL : (mod = (g_regex_unref (mod), NULL)), _tmp41);
					if (inner_error != NULL) {
						mod_name = (g_free (mod_name), NULL);
						pacs = (g_free (pacs), NULL);
						line = (g_free (line), NULL);
						(mod == NULL) ? NULL : (mod = (g_regex_unref (mod), NULL));
						(pac == NULL) ? NULL : (pac = (g_match_info_free (pac), NULL));
						file = (g_free (file), NULL);
						buffer = (g_free (buffer), NULL);
						(reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL));
						(match == NULL) ? NULL : (match = (g_match_info_free (match), NULL));
						name = (g_free (name), NULL);
						version = (g_free (version), NULL);
						url = (g_free (url), NULL);
						goto __catch3_g_error;
						goto __finally3;
					}
					_tmp44 = NULL;
					_tmp42 = NULL;
					if ((_tmp43 = g_regex_match (mod, line, G_REGEX_MATCH_NEWLINE_CR, &_tmp42), pac = (_tmp44 = _tmp42, (pac == NULL) ? NULL : (pac = (g_match_info_free (pac), NULL)), _tmp44), _tmp43)) {
						char* _tmp45;
						_tmp45 = NULL;
						pacs = (_tmp45 = g_match_info_fetch (pac, 1), pacs = (g_free (pacs), NULL), _tmp45);
					} else {
						char* _tmp47;
						const char* _tmp46;
						_tmp47 = NULL;
						_tmp46 = NULL;
						pacs = (_tmp47 = (_tmp46 = line, (_tmp46 == NULL) ? NULL : g_strdup (_tmp46)), pacs = (g_free (pacs), NULL), _tmp47);
					}
				}
				module = vbf_module_new (project, mod_name);
				vbf_project_add_module (project, module);
				if (_vala_strcmp0 (pacs, "") != 0) {
					char** _tmp51;
					gint pkgs_size;
					gint pkgs_length1;
					char** _tmp49;
					char* _tmp48;
					char** _tmp50;
					char** pkgs;
					gint idx;
					_tmp51 = NULL;
					_tmp49 = NULL;
					_tmp48 = NULL;
					_tmp50 = NULL;
					pkgs = (_tmp51 = (_tmp50 = _tmp49 = g_strsplit (_tmp48 = vbf_am_project_manager_normalize_string (self, pacs), " ", 0), _tmp48 = (g_free (_tmp48), NULL), _tmp50), pkgs_length1 = _vala_array_length (_tmp49), pkgs_size = pkgs_length1, _tmp51);
					idx = 0;
					while (pkgs[idx] != NULL) {
						const char* _tmp52;
						char* pkg;
						VbfPackage* package;
						gboolean _tmp53;
						gboolean _tmp54;
						_tmp52 = NULL;
						pkg = (_tmp52 = pkgs[idx], (_tmp52 == NULL) ? NULL : g_strdup (_tmp52));
						package = vbf_package_new (pkg);
						vbf_module_add_package (module, package);
						idx++;
						_tmp53 = FALSE;
						_tmp54 = FALSE;
						if (_vala_strcmp0 (pkgs[idx], ">=") == 0) {
							_tmp54 = TRUE;
						} else {
							_tmp54 = _vala_strcmp0 (pkgs[idx], ">") == 0;
						}
						if (_tmp54) {
							_tmp53 = TRUE;
						} else {
							_tmp53 = _vala_strcmp0 (pkgs[idx], "=") == 0;
						}
						if (_tmp53) {
							char* _tmp56;
							const char* _tmp55;
							VbfConfigNode* node;
							const char* _tmp57;
							char* val;
							VbfConfigNode* _tmp63;
							VbfConfigNode* _tmp62;
							_tmp56 = NULL;
							_tmp55 = NULL;
							package->constraint = (_tmp56 = (_tmp55 = pkgs[idx], (_tmp55 == NULL) ? NULL : g_strdup (_tmp55)), package->constraint = (g_free (package->constraint), NULL), _tmp56);
							idx++;
							node = NULL;
							_tmp57 = NULL;
							val = (_tmp57 = pkgs[idx], (_tmp57 == NULL) ? NULL : g_strdup (_tmp57));
							if (g_str_has_prefix (val, "$")) {
								VbfConfigNode* _tmp59;
								char* _tmp58;
								_tmp59 = NULL;
								_tmp58 = NULL;
								node = (_tmp59 = (VbfConfigNode*) vbf_unresolved_config_node_new (_tmp58 = string_substring (pkgs[idx], (glong) 1, string_get_length (pkgs[idx]) - 1)), (node == NULL) ? NULL : (node = (g_object_unref (node), NULL)), _tmp59);
								_tmp58 = (g_free (_tmp58), NULL);
							} else {
								if (val != NULL) {
									VbfConfigNode* _tmp60;
									_tmp60 = NULL;
									node = (_tmp60 = (VbfConfigNode*) vbf_string_literal_new (pkgs[idx]), (node == NULL) ? NULL : (node = (g_object_unref (node), NULL)), _tmp60);
								} else {
									VbfConfigNode* _tmp61;
									_tmp61 = NULL;
									node = (_tmp61 = (VbfConfigNode*) vbf_string_literal_new (""), (node == NULL) ? NULL : (node = (g_object_unref (node), NULL)), _tmp61);
								}
							}
							_tmp63 = NULL;
							_tmp62 = NULL;
							package->version = (_tmp63 = (_tmp62 = node, (_tmp62 == NULL) ? NULL : g_object_ref (_tmp62)), (package->version == NULL) ? NULL : (package->version = (g_object_unref (package->version), NULL)), _tmp63);
							idx++;
							(node == NULL) ? NULL : (node = (g_object_unref (node), NULL));
							val = (g_free (val), NULL);
						}
						pkg = (g_free (pkg), NULL);
						(package == NULL) ? NULL : (package = (g_object_unref (package), NULL));
					}
					pkgs = (_vala_array_free (pkgs, pkgs_length1, (GDestroyNotify) g_free), NULL);
				}
				g_match_info_next (match, &inner_error);
				if (inner_error != NULL) {
					mod_name = (g_free (mod_name), NULL);
					pacs = (g_free (pacs), NULL);
					line = (g_free (line), NULL);
					(mod == NULL) ? NULL : (mod = (g_regex_unref (mod), NULL));
					(pac == NULL) ? NULL : (pac = (g_match_info_free (pac), NULL));
					(module == NULL) ? NULL : (module = (g_object_unref (module), NULL));
					file = (g_free (file), NULL);
					buffer = (g_free (buffer), NULL);
					(reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL));
					(match == NULL) ? NULL : (match = (g_match_info_free (match), NULL));
					name = (g_free (name), NULL);
					version = (g_free (version), NULL);
					url = (g_free (url), NULL);
					goto __catch3_g_error;
					goto __finally3;
				}
				mod_name = (g_free (mod_name), NULL);
				pacs = (g_free (pacs), NULL);
				line = (g_free (line), NULL);
				(mod == NULL) ? NULL : (mod = (g_regex_unref (mod), NULL));
				(pac == NULL) ? NULL : (pac = (g_match_info_free (pac), NULL));
				(module == NULL) ? NULL : (module = (g_object_unref (module), NULL));
			}
		}
		vbf_am_project_manager_parse_variables (self, project, buffer, NULL, &inner_error);
		if (inner_error != NULL) {
			file = (g_free (file), NULL);
			buffer = (g_free (buffer), NULL);
			(reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL));
			(match == NULL) ? NULL : (match = (g_match_info_free (match), NULL));
			name = (g_free (name), NULL);
			version = (g_free (version), NULL);
			url = (g_free (url), NULL);
			goto __catch3_g_error;
			goto __finally3;
		}
		_tmp64 = NULL;
		vbf_am_project_manager_resolve_variables (self, project, _tmp64 = vbf_project_get_variables (project));
		(_tmp64 == NULL) ? NULL : (_tmp64 = (gee_collection_object_unref (_tmp64), NULL));
		vbf_am_project_manager_resolve_package_variables (self, project);
		/*Extract AC_CONFIG_FILES*/
		_tmp65 = NULL;
		reg = (_tmp65 = g_regex_new ("AC_CONFIG_FILES\\(\\[(.*)\\]\\)", G_REGEX_MULTILINE, 0, &inner_error), (reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL)), _tmp65);
		if (inner_error != NULL) {
			file = (g_free (file), NULL);
			buffer = (g_free (buffer), NULL);
			(reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL));
			(match == NULL) ? NULL : (match = (g_match_info_free (match), NULL));
			name = (g_free (name), NULL);
			version = (g_free (version), NULL);
			url = (g_free (url), NULL);
			goto __catch3_g_error;
			goto __finally3;
		}
		_tmp68 = NULL;
		_tmp66 = NULL;
		res = (_tmp67 = g_regex_match (reg, buffer, G_REGEX_MATCH_NEWLINE_CR, &_tmp66), match = (_tmp68 = _tmp66, (match == NULL) ? NULL : (match = (g_match_info_free (match), NULL)), _tmp68), _tmp67);
		if (!res) {
			GRegex* _tmp69;
			GMatchInfo* _tmp72;
			gboolean _tmp71;
			GMatchInfo* _tmp70;
			_tmp69 = NULL;
			reg = (_tmp69 = g_regex_new ("AC_OUTPUT\\(\\[(.*)\\]\\)", G_REGEX_MULTILINE, 0, &inner_error), (reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL)), _tmp69);
			if (inner_error != NULL) {
				file = (g_free (file), NULL);
				buffer = (g_free (buffer), NULL);
				(reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL));
				(match == NULL) ? NULL : (match = (g_match_info_free (match), NULL));
				name = (g_free (name), NULL);
				version = (g_free (version), NULL);
				url = (g_free (url), NULL);
				goto __catch3_g_error;
				goto __finally3;
			}
			_tmp72 = NULL;
			_tmp70 = NULL;
			res = (_tmp71 = g_regex_match (reg, buffer, G_REGEX_MATCH_NEWLINE_CR, &_tmp70), match = (_tmp72 = _tmp70, (match == NULL) ? NULL : (match = (g_match_info_free (match), NULL)), _tmp72), _tmp71);
		}
		if (res) {
			char* _tmp73;
			char* _tmp74;
			char* tmp;
			char** _tmp76;
			gint makefiles_size;
			gint makefiles_length1;
			char** _tmp75;
			char** makefiles;
			_tmp73 = NULL;
			_tmp74 = NULL;
			tmp = (_tmp74 = vbf_am_project_manager_normalize_string (self, _tmp73 = g_match_info_fetch (match, 1)), _tmp73 = (g_free (_tmp73), NULL), _tmp74);
			_tmp76 = NULL;
			_tmp75 = NULL;
			makefiles = (_tmp76 = _tmp75 = g_strsplit (tmp, " ", 0), makefiles_length1 = _vala_array_length (_tmp75), makefiles_size = makefiles_length1, _tmp76);
			{
				char** makefile_collection;
				int makefile_collection_length1;
				int makefile_it;
				makefile_collection = makefiles;
				makefile_collection_length1 = makefiles_length1;
				for (makefile_it = 0; makefile_it < makefiles_length1; makefile_it = makefile_it + 1) {
					const char* _tmp77;
					char* makefile;
					_tmp77 = NULL;
					makefile = (_tmp77 = makefile_collection[makefile_it], (_tmp77 == NULL) ? NULL : g_strdup (_tmp77));
					{
						vbf_am_project_manager_parse_makefile (self, project, project->id, makefile, &inner_error);
						if (inner_error != NULL) {
							makefile = (g_free (makefile), NULL);
							tmp = (g_free (tmp), NULL);
							makefiles = (_vala_array_free (makefiles, makefiles_length1, (GDestroyNotify) g_free), NULL);
							file = (g_free (file), NULL);
							buffer = (g_free (buffer), NULL);
							(reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL));
							(match == NULL) ? NULL : (match = (g_match_info_free (match), NULL));
							name = (g_free (name), NULL);
							version = (g_free (version), NULL);
							url = (g_free (url), NULL);
							goto __catch3_g_error;
							goto __finally3;
						}
						makefile = (g_free (makefile), NULL);
					}
				}
			}
			tmp = (g_free (tmp), NULL);
			makefiles = (_vala_array_free (makefiles, makefiles_length1, (GDestroyNotify) g_free), NULL);
		}
		vbf_project_setup_file_monitors (project);
		file = (g_free (file), NULL);
		buffer = (g_free (buffer), NULL);
		(reg == NULL) ? NULL : (reg = (g_regex_unref (reg), NULL));
		(match == NULL) ? NULL : (match = (g_match_info_free (match), NULL));
		name = (g_free (name), NULL);
		version = (g_free (version), NULL);
		url = (g_free (url), NULL);
	}
	goto __finally3;
	__catch3_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_critical ("vbfamprojectmanager.vala:181: open: %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			return;
		}
	}
	__finally3:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


static void vbf_am_project_manager_add_vala_source (VbfAmProjectManager* self, VbfGroup* group, VbfTarget* target, VbfConfigNode* source) {
	char* src_name;
	g_return_if_fail (self != NULL);
	g_return_if_fail (group != NULL);
	g_return_if_fail (target != NULL);
	g_return_if_fail (source != NULL);
	src_name = NULL;
	if (VBF_IS_STRING_LITERAL (source)) {
		char* _tmp0;
		VbfSource* src;
		_tmp0 = NULL;
		src_name = (_tmp0 = g_build_filename (group->id, VBF_STRING_LITERAL (source)->data, NULL), src_name = (g_free (src_name), NULL), _tmp0);
		src = vbf_source_new_with_type (target, src_name, VBF_FILE_TYPES_VALA_SOURCE);
		vbf_target_add_source (target, src);
		(src == NULL) ? NULL : (src = (g_object_unref (src), NULL));
	} else {
		if (VBF_IS_VARIABLE (source)) {
			VbfConfigNode* _tmp1;
			_tmp1 = NULL;
			vbf_am_project_manager_add_vala_source (self, group, target, _tmp1 = vbf_variable_get_value (VBF_VARIABLE (source)));
			(_tmp1 == NULL) ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL));
		} else {
			if (VBF_IS_CONFIG_NODE_LIST (source)) {
				{
					GeeList* _tmp2;
					GeeIterator* _tmp3;
					GeeIterator* _item_it;
					_tmp2 = NULL;
					_tmp3 = NULL;
					_item_it = (_tmp3 = gee_iterable_iterator ((GeeIterable*) (_tmp2 = vbf_config_node_list_get_values (VBF_CONFIG_NODE_LIST (source)))), (_tmp2 == NULL) ? NULL : (_tmp2 = (gee_collection_object_unref (_tmp2), NULL)), _tmp3);
					while (gee_iterator_next (_item_it)) {
						VbfConfigNode* item;
						item = (VbfConfigNode*) gee_iterator_get (_item_it);
						if (VBF_IS_STRING_LITERAL (item)) {
							char* _tmp4;
							VbfSource* src;
							_tmp4 = NULL;
							src_name = (_tmp4 = g_build_filename (group->id, VBF_STRING_LITERAL (item)->data, NULL), src_name = (g_free (src_name), NULL), _tmp4);
							src = vbf_source_new_with_type (target, src_name, VBF_FILE_TYPES_VALA_SOURCE);
							vbf_target_add_source (target, src);
							(src == NULL) ? NULL : (src = (g_object_unref (src), NULL));
						} else {
							if (VBF_IS_VARIABLE (item)) {
								VbfConfigNode* _tmp5;
								_tmp5 = NULL;
								vbf_am_project_manager_add_vala_source (self, group, target, _tmp5 = vbf_variable_get_value (VBF_VARIABLE (item)));
								(_tmp5 == NULL) ? NULL : (_tmp5 = (g_object_unref (_tmp5), NULL));
							} else {
								if (VBF_IS_CONFIG_NODE_LIST (item)) {
									vbf_am_project_manager_add_vala_source (self, group, target, item);
								}
							}
						}
						(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
					}
					(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
				}
			} else {
				g_warning ("vbfamprojectmanager.vala:208: add_vala_source: unsupported source type");
			}
		}
	}
	src_name = (g_free (src_name), NULL);
}


static void vbf_am_project_manager_add_vala_sources (VbfAmProjectManager* self, VbfGroup* group, VbfTarget* target) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (group != NULL);
	g_return_if_fail (target != NULL);
	{
		GeeList* _tmp0;
		GeeIterator* _tmp1;
		GeeIterator* _variable_it;
		_tmp0 = NULL;
		_tmp1 = NULL;
		_variable_it = (_tmp1 = gee_iterable_iterator ((GeeIterable*) (_tmp0 = vbf_group_get_variables (group))), (_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL)), _tmp1);
		while (gee_iterator_next (_variable_it)) {
			VbfVariable* variable;
			variable = (VbfVariable*) gee_iterator_get (_variable_it);
			if (_vala_strcmp0 (variable->name, target->name) == 0) {
				VbfConfigNode* val;
				val = vbf_variable_get_value (variable);
				vbf_am_project_manager_add_vala_source (self, group, target, val);
				(val == NULL) ? NULL : (val = (g_object_unref (val), NULL));
				(variable == NULL) ? NULL : (variable = (g_object_unref (variable), NULL));
				break;
			}
			(variable == NULL) ? NULL : (variable = (g_object_unref (variable), NULL));
		}
		(_variable_it == NULL) ? NULL : (_variable_it = (gee_collection_object_unref (_variable_it), NULL));
	}
}


static void vbf_am_project_manager_add_targets (VbfAmProjectManager* self, VbfGroup* group, VbfConfigNode* node, VbfTargetTypes type) {
	VbfTarget* target;
	g_return_if_fail (self != NULL);
	g_return_if_fail (group != NULL);
	g_return_if_fail (node != NULL);
	target = NULL;
	if (VBF_IS_STRING_LITERAL (node)) {
		VbfTarget* _tmp0;
		_tmp0 = NULL;
		target = (_tmp0 = vbf_target_new (group, type, VBF_STRING_LITERAL (node)->data), (target == NULL) ? NULL : (target = (g_object_unref (target), NULL)), _tmp0);
		vbf_group_add_target (group, target);
		vbf_am_project_manager_add_vala_sources (self, group, target);
	} else {
		{
			GeeIterator* _item_it;
			_item_it = gee_iterable_iterator ((GeeIterable*) VBF_CONFIG_NODE_LIST (node)->values);
			while (gee_iterator_next (_item_it)) {
				VbfConfigNode* item;
				item = (VbfConfigNode*) gee_iterator_get (_item_it);
				if (VBF_IS_STRING_LITERAL (item)) {
					const char* _tmp1;
					char* data;
					_tmp1 = NULL;
					data = (_tmp1 = VBF_STRING_LITERAL (item)->data, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
					if (g_str_has_suffix (data, "stamp")) {
						VbfTarget* _tmp2;
						_tmp2 = NULL;
						target = (_tmp2 = vbf_target_new (group, type, data), (target == NULL) ? NULL : (target = (g_object_unref (target), NULL)), _tmp2);
						vbf_group_add_target (group, target);
						vbf_am_project_manager_add_vala_sources (self, group, target);
					}
					data = (g_free (data), NULL);
				}
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			}
			(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
		}
	}
	(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
}


static void vbf_am_project_manager_add_data_files (VbfAmProjectManager* self, VbfGroup* group, VbfTarget* target, VbfConfigNode* file) {
	char* name;
	g_return_if_fail (self != NULL);
	g_return_if_fail (group != NULL);
	g_return_if_fail (target != NULL);
	g_return_if_fail (file != NULL);
	name = NULL;
	if (VBF_IS_STRING_LITERAL (file)) {
		char* _tmp0;
		VbfFile* f;
		_tmp0 = NULL;
		name = (_tmp0 = g_build_filename (group->id, VBF_STRING_LITERAL (file)->data, NULL), name = (g_free (name), NULL), _tmp0);
		f = vbf_file_new_with_type (target, name, VBF_FILE_TYPES_DATA);
		vbf_target_add_file (target, f);
		(f == NULL) ? NULL : (f = (g_object_unref (f), NULL));
	} else {
		if (VBF_IS_VARIABLE (file)) {
			VbfConfigNode* _tmp1;
			_tmp1 = NULL;
			vbf_am_project_manager_add_data_files (self, group, target, _tmp1 = vbf_variable_get_value (VBF_VARIABLE (file)));
			(_tmp1 == NULL) ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL));
		} else {
			if (VBF_IS_CONFIG_NODE_LIST (file)) {
				{
					GeeList* _tmp2;
					GeeIterator* _tmp3;
					GeeIterator* _item_it;
					_tmp2 = NULL;
					_tmp3 = NULL;
					_item_it = (_tmp3 = gee_iterable_iterator ((GeeIterable*) (_tmp2 = vbf_config_node_list_get_values (VBF_CONFIG_NODE_LIST (file)))), (_tmp2 == NULL) ? NULL : (_tmp2 = (gee_collection_object_unref (_tmp2), NULL)), _tmp3);
					while (gee_iterator_next (_item_it)) {
						VbfConfigNode* item;
						item = (VbfConfigNode*) gee_iterator_get (_item_it);
						if (VBF_IS_STRING_LITERAL (item)) {
							char* _tmp4;
							VbfFile* f;
							_tmp4 = NULL;
							name = (_tmp4 = g_build_filename (group->id, VBF_STRING_LITERAL (item)->data, NULL), name = (g_free (name), NULL), _tmp4);
							f = vbf_file_new_with_type (target, name, VBF_FILE_TYPES_DATA);
							vbf_target_add_file (target, f);
							(f == NULL) ? NULL : (f = (g_object_unref (f), NULL));
						} else {
							if (VBF_IS_VARIABLE (item)) {
								VbfConfigNode* _tmp5;
								_tmp5 = NULL;
								vbf_am_project_manager_add_data_files (self, group, target, _tmp5 = vbf_variable_get_value (VBF_VARIABLE (item)));
								(_tmp5 == NULL) ? NULL : (_tmp5 = (g_object_unref (_tmp5), NULL));
							} else {
								if (VBF_IS_CONFIG_NODE_LIST (item)) {
									vbf_am_project_manager_add_data_files (self, group, target, item);
								}
							}
						}
						(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
					}
					(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
				}
			} else {
				g_warning ("vbfamprojectmanager.vala:267: add_vala_source: unsupported source type");
			}
		}
	}
	name = (g_free (name), NULL);
}


static void vbf_am_project_manager_parse_targets (VbfAmProjectManager* self, VbfGroup* group) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (group != NULL);
	{
		GeeList* _tmp0;
		GeeIterator* _tmp1;
		GeeIterator* _variable_it;
		_tmp0 = NULL;
		_tmp1 = NULL;
		_variable_it = (_tmp1 = gee_iterable_iterator ((GeeIterable*) (_tmp0 = vbf_group_get_variables (group))), (_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL)), _tmp1);
		while (gee_iterator_next (_variable_it)) {
			VbfVariable* variable;
			variable = (VbfVariable*) gee_iterator_get (_variable_it);
			if (g_str_has_suffix (variable->name, "_PROGRAMS")) {
				VbfConfigNode* _tmp2;
				_tmp2 = NULL;
				vbf_am_project_manager_add_targets (self, group, _tmp2 = vbf_variable_get_value (variable), VBF_TARGET_TYPES_PROGRAM);
				(_tmp2 == NULL) ? NULL : (_tmp2 = (g_object_unref (_tmp2), NULL));
			} else {
				gboolean _tmp3;
				_tmp3 = FALSE;
				if (g_str_has_suffix (variable->name, "_LTLIBRARIES")) {
					_tmp3 = TRUE;
				} else {
					_tmp3 = g_str_has_suffix (variable->name, "_LIBRARIES");
				}
				if (_tmp3) {
					VbfConfigNode* _tmp4;
					_tmp4 = NULL;
					vbf_am_project_manager_add_targets (self, group, _tmp4 = vbf_variable_get_value (variable), VBF_TARGET_TYPES_LIBRARY);
					(_tmp4 == NULL) ? NULL : (_tmp4 = (g_object_unref (_tmp4), NULL));
				} else {
					if (g_str_has_suffix (variable->name, "_DATA")) {
						VbfTarget* target;
						VbfConfigNode* _tmp5;
						target = vbf_target_new (group, VBF_TARGET_TYPES_DATA, "data");
						_tmp5 = NULL;
						vbf_am_project_manager_add_data_files (self, group, target, _tmp5 = vbf_variable_get_value (variable));
						(_tmp5 == NULL) ? NULL : (_tmp5 = (g_object_unref (_tmp5), NULL));
						vbf_group_add_target (group, target);
						(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
					} else {
						if (_vala_strcmp0 (variable->name, "BUILT_SOURCES") == 0) {
							VbfConfigNode* _tmp6;
							_tmp6 = NULL;
							vbf_am_project_manager_add_targets (self, group, _tmp6 = vbf_variable_get_value (variable), VBF_TARGET_TYPES_BUILT_SOURCES);
							(_tmp6 == NULL) ? NULL : (_tmp6 = (g_object_unref (_tmp6), NULL));
						}
					}
				}
			}
			(variable == NULL) ? NULL : (variable = (g_object_unref (variable), NULL));
		}
		(_variable_it == NULL) ? NULL : (_variable_it = (gee_collection_object_unref (_variable_it), NULL));
	}
}


static void vbf_am_project_manager_parse_variables (VbfAmProjectManager* self, VbfProject* project, const char* buffer, VbfGroup* group, GError** error) {
	char** _tmp1;
	gint lines_size;
	gint lines_length1;
	char** _tmp0;
	char** lines;
	char* tmp;
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	g_return_if_fail (buffer != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	lines = (_tmp1 = _tmp0 = g_strsplit (buffer, "\n", 0), lines_length1 = _vala_array_length (_tmp0), lines_size = lines_length1, _tmp1);
	tmp = NULL;
	{
		char** line_collection;
		int line_collection_length1;
		int line_it;
		line_collection = lines;
		line_collection_length1 = lines_length1;
		for (line_it = 0; line_it < lines_length1; line_it = line_it + 1) {
			const char* _tmp43;
			char* line;
			_tmp43 = NULL;
			line = (_tmp43 = line_collection[line_it], (_tmp43 == NULL) ? NULL : g_strdup (_tmp43));
			{
				gboolean _tmp7;
				gboolean append;
				char** _tmp10;
				gint toks_size;
				gint toks_length1;
				char** _tmp9;
				char** toks;
				char** _tmp16;
				gint lhs_size;
				gint lhs_length1;
				char** _tmp15;
				char** lhs;
				char** _tmp18;
				gint rhs_size;
				gint rhs_length1;
				char** _tmp17;
				char** rhs;
				char* _tmp42;
				if (tmp == NULL) {
					char* _tmp2;
					_tmp2 = NULL;
					tmp = (_tmp2 = vbf_am_project_manager_normalize_string (self, line), tmp = (g_free (tmp), NULL), _tmp2);
				} else {
					char* _tmp5;
					char* _tmp4;
					char* _tmp3;
					_tmp5 = NULL;
					_tmp4 = NULL;
					_tmp3 = NULL;
					tmp = (_tmp5 = g_strconcat (tmp, _tmp4 = g_strconcat (" ", _tmp3 = vbf_am_project_manager_normalize_string (self, line), NULL), NULL), tmp = (g_free (tmp), NULL), _tmp5);
					_tmp4 = (g_free (_tmp4), NULL);
					_tmp3 = (g_free (_tmp3), NULL);
				}
				if (g_str_has_suffix (tmp, "\\")) {
					char* _tmp6;
					_tmp6 = NULL;
					tmp = (_tmp6 = string_substring (tmp, (glong) 0, string_get_length (tmp) - 1), tmp = (g_free (tmp), NULL), _tmp6);
					line = (g_free (line), NULL);
					continue;
				}
				_tmp7 = FALSE;
				if (g_str_has_prefix (tmp, "#")) {
					_tmp7 = TRUE;
				} else {
					_tmp7 = _vala_strcmp0 (tmp, "") == 0;
				}
				/*line continuation grab the next line*/
				if (_tmp7) {
					char* _tmp8;
					_tmp8 = NULL;
					tmp = (_tmp8 = NULL, tmp = (g_free (tmp), NULL), _tmp8);
					line = (g_free (line), NULL);
					continue;
				}
				append = FALSE;
				_tmp10 = NULL;
				_tmp9 = NULL;
				toks = (_tmp10 = _tmp9 = g_strsplit (tmp, "=", 2), toks_length1 = _vala_array_length (_tmp9), toks_size = toks_length1, _tmp10);
				if (toks[1] == NULL) {
					char** _tmp12;
					char** _tmp11;
					_tmp12 = NULL;
					_tmp11 = NULL;
					toks = (_tmp12 = _tmp11 = g_strsplit (tmp, ":", 2), toks = (_vala_array_free (toks, toks_length1, (GDestroyNotify) g_free), NULL), toks_length1 = _vala_array_length (_tmp11), toks_size = toks_length1, _tmp12);
				}
				if (toks[1] == NULL) {
					char* _tmp13;
					_tmp13 = NULL;
					tmp = (_tmp13 = NULL, tmp = (g_free (tmp), NULL), _tmp13);
					line = (g_free (line), NULL);
					toks = (_vala_array_free (toks, toks_length1, (GDestroyNotify) g_free), NULL);
					continue;
				}
				/*unrecognized pattern
				check if is an append to var*/
				if (g_str_has_suffix (toks[0], "+")) {
					char* _tmp14;
					append = TRUE;
					_tmp14 = NULL;
					toks[0] = (_tmp14 = string_substring (toks[0], (glong) 0, string_get_length (toks[0]) - 1), toks[0] = (g_free (toks[0]), NULL), _tmp14);
				}
				_tmp16 = NULL;
				_tmp15 = NULL;
				lhs = (_tmp16 = _tmp15 = g_strsplit (toks[0], " ", 0), lhs_length1 = _vala_array_length (_tmp15), lhs_size = lhs_length1, _tmp16);
				_tmp18 = NULL;
				_tmp17 = NULL;
				rhs = (_tmp18 = _tmp17 = g_strsplit (toks[1], " ", 0), rhs_length1 = _vala_array_length (_tmp17), rhs_size = rhs_length1, _tmp18);
				{
					char** lh_collection;
					int lh_collection_length1;
					int lh_it;
					lh_collection = lhs;
					lh_collection_length1 = lhs_length1;
					for (lh_it = 0; lh_it < lhs_length1; lh_it = lh_it + 1) {
						const char* _tmp41;
						char* lh;
						_tmp41 = NULL;
						lh = (_tmp41 = lh_collection[lh_it], (_tmp41 == NULL) ? NULL : g_strdup (_tmp41));
						{
							gboolean _tmp19;
							const char* _tmp20;
							char* name;
							VbfVariable* variable;
							VbfConfigNode* data;
							VbfConfigNode* _tmp40;
							VbfConfigNode* _tmp39;
							_tmp19 = FALSE;
							if (lh == NULL) {
								_tmp19 = TRUE;
							} else {
								_tmp19 = _vala_strcmp0 (lh, "") == 0;
							}
							if (_tmp19) {
								lh = (g_free (lh), NULL);
								continue;
							}
							_tmp20 = NULL;
							name = (_tmp20 = g_strstrip (lh), (_tmp20 == NULL) ? NULL : g_strdup (_tmp20));
							variable = vbf_variable_new (name, (VbfConfigNode*) project);
							data = NULL;
							if (rhs[1] == NULL) {
								const char* _tmp21;
								char* val;
								_tmp21 = NULL;
								val = (_tmp21 = g_strstrip (rhs[0]), (_tmp21 == NULL) ? NULL : g_strdup (_tmp21));
								if (g_str_has_prefix (val, "$")) {
									char* _tmp22;
									char* _tmp24;
									char* _tmp23;
									VbfConfigNode* _tmp25;
									_tmp22 = NULL;
									val = (_tmp22 = string_substring (val, (glong) 1, string_get_length (val) - 1), val = (g_free (val), NULL), _tmp22);
									_tmp24 = NULL;
									_tmp23 = NULL;
									val = (_tmp24 = string_replace (_tmp23 = string_replace (val, "(", ""), ")", ""), val = (g_free (val), NULL), _tmp24);
									_tmp23 = (g_free (_tmp23), NULL);
									_tmp25 = NULL;
									data = (_tmp25 = (VbfConfigNode*) vbf_unresolved_config_node_new (val), (data == NULL) ? NULL : (data = (g_object_unref (data), NULL)), _tmp25);
								} else {
									if (val != NULL) {
										VbfConfigNode* _tmp26;
										_tmp26 = NULL;
										data = (_tmp26 = (VbfConfigNode*) vbf_string_literal_new (val), (data == NULL) ? NULL : (data = (g_object_unref (data), NULL)), _tmp26);
									} else {
										VbfConfigNode* _tmp27;
										_tmp27 = NULL;
										data = (_tmp27 = (VbfConfigNode*) vbf_string_literal_new (""), (data == NULL) ? NULL : (data = (g_object_unref (data), NULL)), _tmp27);
									}
								}
								val = (g_free (val), NULL);
							} else {
								VbfConfigNodeList* items;
								VbfConfigNode* _tmp38;
								VbfConfigNode* _tmp37;
								items = vbf_config_node_list_new ();
								{
									char** rh_collection;
									int rh_collection_length1;
									int rh_it;
									rh_collection = rhs;
									rh_collection_length1 = rhs_length1;
									for (rh_it = 0; rh_it < rhs_length1; rh_it = rh_it + 1) {
										const char* _tmp36;
										char* rh;
										_tmp36 = NULL;
										rh = (_tmp36 = rh_collection[rh_it], (_tmp36 == NULL) ? NULL : g_strdup (_tmp36));
										{
											gboolean _tmp28;
											_tmp28 = FALSE;
											if (rh != NULL) {
												_tmp28 = _vala_strcmp0 (g_strstrip (rh), "") != 0;
											} else {
												_tmp28 = FALSE;
											}
											if (_tmp28) {
												VbfConfigNode* node;
												const char* _tmp29;
												char* val;
												node = NULL;
												_tmp29 = NULL;
												val = (_tmp29 = g_strstrip (rh), (_tmp29 == NULL) ? NULL : g_strdup (_tmp29));
												if (g_str_has_prefix (val, "$")) {
													char* _tmp30;
													char* _tmp32;
													char* _tmp31;
													VbfConfigNode* _tmp33;
													_tmp30 = NULL;
													val = (_tmp30 = string_substring (val, (glong) 1, string_get_length (val) - 1), val = (g_free (val), NULL), _tmp30);
													_tmp32 = NULL;
													_tmp31 = NULL;
													val = (_tmp32 = string_replace (_tmp31 = string_replace (val, "(", ""), ")", ""), val = (g_free (val), NULL), _tmp32);
													_tmp31 = (g_free (_tmp31), NULL);
													_tmp33 = NULL;
													node = (_tmp33 = (VbfConfigNode*) vbf_unresolved_config_node_new (val), (node == NULL) ? NULL : (node = (g_object_unref (node), NULL)), _tmp33);
												} else {
													if (val != NULL) {
														VbfConfigNode* _tmp34;
														_tmp34 = NULL;
														node = (_tmp34 = (VbfConfigNode*) vbf_string_literal_new (val), (node == NULL) ? NULL : (node = (g_object_unref (node), NULL)), _tmp34);
													} else {
														VbfConfigNode* _tmp35;
														g_critical ("vbfamprojectmanager.vala:368: %s is null", rh);
														_tmp35 = NULL;
														node = (_tmp35 = (VbfConfigNode*) vbf_string_literal_new (""), (node == NULL) ? NULL : (node = (g_object_unref (node), NULL)), _tmp35);
													}
												}
												vbf_config_node_list_add_value (items, node);
												(node == NULL) ? NULL : (node = (g_object_unref (node), NULL));
												val = (g_free (val), NULL);
											}
											rh = (g_free (rh), NULL);
										}
									}
								}
								_tmp38 = NULL;
								_tmp37 = NULL;
								data = (_tmp38 = (_tmp37 = (VbfConfigNode*) items, (_tmp37 == NULL) ? NULL : g_object_ref (_tmp37)), (data == NULL) ? NULL : (data = (g_object_unref (data), NULL)), _tmp38);
								(items == NULL) ? NULL : (items = (g_object_unref (items), NULL));
							}
							_tmp40 = NULL;
							_tmp39 = NULL;
							variable->data = (_tmp40 = (_tmp39 = data, (_tmp39 == NULL) ? NULL : g_object_ref (_tmp39)), (variable->data == NULL) ? NULL : (variable->data = (g_object_unref (variable->data), NULL)), _tmp40);
							if (group != NULL) {
								vbf_group_add_variable (group, variable);
							} else {
								vbf_project_add_variable (project, variable);
							}
							lh = (g_free (lh), NULL);
							name = (g_free (name), NULL);
							(variable == NULL) ? NULL : (variable = (g_object_unref (variable), NULL));
							(data == NULL) ? NULL : (data = (g_object_unref (data), NULL));
						}
					}
				}
				/*
				//this regex matches:
				//(spaces)(variable_name):(list variable names or version numbers)
				//(spaces)(variable_name)=(list variable names or version numbers)
				//(spaces)(variable_name)+=(list variable names or version numbers)
				Regex reg_var = new GLib.Regex ("^\s*([A-Za-z_$$\(]+[ A-Za-z_0-9\.$$\)\-]*)(=)([A-Za-z_0-9]*[ \.A-Za-z_0-9$$\(\)/\-:=]*)", RegexCompileFlags.OPTIMIZE);
				Regex reg_rule = new GLib.Regex ("^\s*([A-Za-z_$$\(]+[ A-Za-z_0-9\.$$\)\-]*)(:)([A-Za-z_0-9]*[ \.A-Za-z_0-9$$\(\)/\-:=]*)", RegexCompileFlags.OPTIMIZE);
				
				MatchInfo match;
				tmp = normalize_string (tmp);
				tmp = tmp.replace ("= ", "="); //hack: the regular expression has a bug
				tmp = tmp.replace (" =", "=");
				
				bool res = reg_var.match (tmp, 0, out match);
				if (!res)
				res = reg_rule.match (tmp, 0, out match);
				if (res) {
				debug ("found %s, values %s", match.fetch (0), match.fetch (3));
				
				string rule_name = normalize_string (match.fetch(1));
				string[] names = rule_name.split (" ");
				string values = normalize_string (match.fetch(3));
				string[] tmps = values.split (" ");
				debug ("FOUND VARIABLE: %s %s", rule_name, values);
				
				foreach (string name in names) {
				var variable = new Variable(name, project);
				ConfigNode data;
				
				if (tmps.length == 1) {
				string val = normalize_string (tmps[0]);
				
				if (val.has_prefix ("$")) {
				val = val.substring (1, val.length - 1);
				val = val.replace ("(", "").replace (")", "");
				data = new UnresolvedConfigNode (val);
				} else if (val != null){
				data = new StringLiteral (val);
				} else {
				data = new StringLiteral ("");
				}
				} else {
				ConfigNodeList items = new ConfigNodeList ();
				
				foreach (string tmp in tmps) {
				if (tmp != null && tmp.chomp () != "") {
				ConfigNode node;
				string val = normalize_string (tmp);
				
				if (val.has_prefix ("$")) {
				val = val.substring (1, val.length - 1);
				val = val.replace ("(", "").replace (")", "");
				node = new UnresolvedConfigNode (val);
				} else if (val != null){
				node = new StringLiteral (val);
				} else {
				critical ("%s is null", tmp);
				node = new StringLiteral ("");
				}
				
				items.add_value (node);
				}
				}
				data = items;
				}
				variable.data = data;
				if (variable.name == "BUILT_SOURCES") {
				debug ("BS: VALUES %s", variable.data.to_string ());
				}
				if (group != null) {
				group.add_variable (variable);
				} else {
				project.add_variable (variable);
				}
				}
				}
				*/
				_tmp42 = NULL;
				tmp = (_tmp42 = NULL, tmp = (g_free (tmp), NULL), _tmp42);
				line = (g_free (line), NULL);
				toks = (_vala_array_free (toks, toks_length1, (GDestroyNotify) g_free), NULL);
				lhs = (_vala_array_free (lhs, lhs_length1, (GDestroyNotify) g_free), NULL);
				rhs = (_vala_array_free (rhs, rhs_length1, (GDestroyNotify) g_free), NULL);
			}
		}
	}
	lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL);
	tmp = (g_free (tmp), NULL);
}


/*restart from a new line*/
static VbfVariable* vbf_am_project_manager_resolve_variable (VbfAmProjectManager* self, const char* variable_name, GeeList* variables) {
	VbfVariable* variable;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (variable_name != NULL, NULL);
	g_return_val_if_fail (variables != NULL, NULL);
	variable = NULL;
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator ((GeeIterable*) variables);
		while (gee_iterator_next (_item_it)) {
			VbfVariable* item;
			item = (VbfVariable*) gee_iterator_get (_item_it);
			if (_vala_strcmp0 (variable_name, item->name) == 0) {
				VbfVariable* _tmp1;
				VbfVariable* _tmp0;
				_tmp1 = NULL;
				_tmp0 = NULL;
				variable = (_tmp1 = (_tmp0 = item, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (variable == NULL) ? NULL : (variable = (g_object_unref (variable), NULL)), _tmp1);
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
				break;
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
	}
	return variable;
}


static void vbf_am_project_manager_resolve_package_variables (VbfAmProjectManager* self, VbfProject* project) {
	GeeList* variables;
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	/*resolve package version variables*/
	variables = vbf_project_get_variables (project);
	{
		GeeList* _tmp0;
		GeeIterator* _tmp1;
		GeeIterator* _module_it;
		_tmp0 = NULL;
		_tmp1 = NULL;
		_module_it = (_tmp1 = gee_iterable_iterator ((GeeIterable*) (_tmp0 = vbf_project_get_modules (project))), (_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL)), _tmp1);
		while (gee_iterator_next (_module_it)) {
			VbfModule* module;
			module = (VbfModule*) gee_iterator_get (_module_it);
			{
				GeeList* _tmp2;
				GeeIterator* _tmp3;
				GeeIterator* _package_it;
				_tmp2 = NULL;
				_tmp3 = NULL;
				_package_it = (_tmp3 = gee_iterable_iterator ((GeeIterable*) (_tmp2 = vbf_module_get_packages (module))), (_tmp2 == NULL) ? NULL : (_tmp2 = (gee_collection_object_unref (_tmp2), NULL)), _tmp3);
				while (gee_iterator_next (_package_it)) {
					VbfPackage* package;
					package = (VbfPackage*) gee_iterator_get (_package_it);
					if (VBF_IS_UNRESOLVED_CONFIG_NODE (package->version)) {
						const char* _tmp4;
						char* name;
						VbfVariable* target_variable;
						_tmp4 = NULL;
						name = (_tmp4 = VBF_UNRESOLVED_CONFIG_NODE (package->version)->name, (_tmp4 == NULL) ? NULL : g_strdup (_tmp4));
						target_variable = vbf_am_project_manager_resolve_variable (self, name, variables);
						if (target_variable != NULL) {
							VbfConfigNode* _tmp6;
							VbfConfigNode* _tmp5;
							_tmp6 = NULL;
							_tmp5 = NULL;
							package->version = (_tmp6 = (_tmp5 = target_variable->data, (_tmp5 == NULL) ? NULL : g_object_ref (_tmp5)), (package->version == NULL) ? NULL : (package->version = (g_object_unref (package->version), NULL)), _tmp6);
						}
						name = (g_free (name), NULL);
						(target_variable == NULL) ? NULL : (target_variable = (g_object_unref (target_variable), NULL));
					}
					(package == NULL) ? NULL : (package = (g_object_unref (package), NULL));
				}
				(_package_it == NULL) ? NULL : (_package_it = (gee_collection_object_unref (_package_it), NULL));
			}
			(module == NULL) ? NULL : (module = (g_object_unref (module), NULL));
		}
		(_module_it == NULL) ? NULL : (_module_it = (gee_collection_object_unref (_module_it), NULL));
	}
	(variables == NULL) ? NULL : (variables = (gee_collection_object_unref (variables), NULL));
}


static void vbf_am_project_manager_resolve_variables (VbfAmProjectManager* self, VbfProject* project, GeeList* variables) {
	VbfVariable* target_variable;
	char* name;
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	g_return_if_fail (variables != NULL);
	target_variable = NULL;
	name = NULL;
	/*resolve variables*/
	{
		GeeIterator* _variable_it;
		/*resolve variables*/
		_variable_it = gee_iterable_iterator ((GeeIterable*) variables);
		/*resolve variables*/
		while (gee_iterator_next (_variable_it)) {
			VbfVariable* variable;
			/*resolve variables*/
			variable = (VbfVariable*) gee_iterator_get (_variable_it);
			if (VBF_IS_UNRESOLVED_CONFIG_NODE (variable->data)) {
				char* _tmp1;
				const char* _tmp0;
				VbfVariable* _tmp2;
				_tmp1 = NULL;
				_tmp0 = NULL;
				name = (_tmp1 = (_tmp0 = VBF_UNRESOLVED_CONFIG_NODE (variable->data)->name, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), name = (g_free (name), NULL), _tmp1);
				_tmp2 = NULL;
				target_variable = (_tmp2 = vbf_am_project_manager_resolve_variable (self, name, variables), (target_variable == NULL) ? NULL : (target_variable = (g_object_unref (target_variable), NULL)), _tmp2);
				if (target_variable == NULL) {
					VbfVariable* _tmp4;
					GeeList* _tmp3;
					_tmp4 = NULL;
					_tmp3 = NULL;
					target_variable = (_tmp4 = vbf_am_project_manager_resolve_variable (self, name, _tmp3 = vbf_project_get_variables (project)), (target_variable == NULL) ? NULL : (target_variable = (g_object_unref (target_variable), NULL)), _tmp4);
					(_tmp3 == NULL) ? NULL : (_tmp3 = (gee_collection_object_unref (_tmp3), NULL));
				}
				if (target_variable != NULL) {
					VbfConfigNode* _tmp6;
					VbfConfigNode* _tmp5;
					_tmp6 = NULL;
					_tmp5 = NULL;
					variable->data = (_tmp6 = (_tmp5 = (VbfConfigNode*) target_variable, (_tmp5 == NULL) ? NULL : g_object_ref (_tmp5)), (variable->data == NULL) ? NULL : (variable->data = (g_object_unref (variable->data), NULL)), _tmp6);
					((VbfConfigNode*) variable)->parent = (VbfConfigNode*) target_variable;
					vbf_variable_add_child (target_variable, variable);
				}
			} else {
				if (VBF_IS_CONFIG_NODE_LIST (variable->data)) {
					VbfConfigNodeList* _tmp7;
					VbfConfigNodeList* datas;
					GeeList* resolved_nodes;
					_tmp7 = NULL;
					datas = (_tmp7 = VBF_CONFIG_NODE_LIST (variable->data), (_tmp7 == NULL) ? NULL : g_object_ref (_tmp7));
					resolved_nodes = (GeeList*) gee_array_list_new (VBF_TYPE_CONFIG_NODE_PAIR, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
					{
						GeeList* _tmp8;
						GeeIterator* _tmp9;
						GeeIterator* _data_it;
						_tmp8 = NULL;
						_tmp9 = NULL;
						_data_it = (_tmp9 = gee_iterable_iterator ((GeeIterable*) (_tmp8 = vbf_config_node_list_get_values (datas))), (_tmp8 == NULL) ? NULL : (_tmp8 = (gee_collection_object_unref (_tmp8), NULL)), _tmp9);
						while (gee_iterator_next (_data_it)) {
							VbfConfigNode* data;
							data = (VbfConfigNode*) gee_iterator_get (_data_it);
							if (VBF_IS_UNRESOLVED_CONFIG_NODE (data)) {
								char* _tmp11;
								const char* _tmp10;
								VbfVariable* _tmp12;
								_tmp11 = NULL;
								_tmp10 = NULL;
								name = (_tmp11 = (_tmp10 = VBF_UNRESOLVED_CONFIG_NODE (data)->name, (_tmp10 == NULL) ? NULL : g_strdup (_tmp10)), name = (g_free (name), NULL), _tmp11);
								_tmp12 = NULL;
								target_variable = (_tmp12 = vbf_am_project_manager_resolve_variable (self, name, variables), (target_variable == NULL) ? NULL : (target_variable = (g_object_unref (target_variable), NULL)), _tmp12);
								if (target_variable == NULL) {
									VbfVariable* _tmp14;
									GeeList* _tmp13;
									_tmp14 = NULL;
									_tmp13 = NULL;
									target_variable = (_tmp14 = vbf_am_project_manager_resolve_variable (self, name, _tmp13 = vbf_project_get_variables (project)), (target_variable == NULL) ? NULL : (target_variable = (g_object_unref (target_variable), NULL)), _tmp14);
									(_tmp13 == NULL) ? NULL : (_tmp13 = (gee_collection_object_unref (_tmp13), NULL));
								}
								if (target_variable != NULL) {
									if (target_variable->data != NULL) {
										VbfVariable* child;
										VbfConfigNode* _tmp16;
										VbfConfigNode* _tmp15;
										VbfConfigNodePair* _tmp17;
										child = vbf_variable_new (name, (VbfConfigNode*) target_variable);
										_tmp16 = NULL;
										_tmp15 = NULL;
										child->data = (_tmp16 = (_tmp15 = (VbfConfigNode*) target_variable, (_tmp15 == NULL) ? NULL : g_object_ref (_tmp15)), (child->data == NULL) ? NULL : (child->data = (g_object_unref (child->data), NULL)), _tmp16);
										vbf_variable_add_child (target_variable, child);
										_tmp17 = NULL;
										gee_collection_add ((GeeCollection*) resolved_nodes, _tmp17 = vbf_config_node_pair_new (data, (VbfConfigNode*) child));
										(_tmp17 == NULL) ? NULL : (_tmp17 = (g_object_unref (_tmp17), NULL));
										(child == NULL) ? NULL : (child = (g_object_unref (child), NULL));
									} else {
										VbfConfigNodePair* _tmp18;
										_tmp18 = NULL;
										gee_collection_add ((GeeCollection*) resolved_nodes, _tmp18 = vbf_config_node_pair_new (data, NULL));
										(_tmp18 == NULL) ? NULL : (_tmp18 = (g_object_unref (_tmp18), NULL));
									}
								}
							}
							(data == NULL) ? NULL : (data = (g_object_unref (data), NULL));
						}
						(_data_it == NULL) ? NULL : (_data_it = (gee_collection_object_unref (_data_it), NULL));
					}
					/*modify the references*/
					{
						GeeIterator* _resolved_node_it;
						/*modify the references*/
						_resolved_node_it = gee_iterable_iterator ((GeeIterable*) resolved_nodes);
						/*modify the references*/
						while (gee_iterator_next (_resolved_node_it)) {
							VbfConfigNodePair* resolved_node;
							/*modify the references*/
							resolved_node = (VbfConfigNodePair*) gee_iterator_get (_resolved_node_it);
							vbf_config_node_list_replace_config_node (datas, resolved_node->source, resolved_node->destination);
							(resolved_node == NULL) ? NULL : (resolved_node = (g_object_unref (resolved_node), NULL));
						}
						(_resolved_node_it == NULL) ? NULL : (_resolved_node_it = (gee_collection_object_unref (_resolved_node_it), NULL));
					}
					(datas == NULL) ? NULL : (datas = (g_object_unref (datas), NULL));
					(resolved_nodes == NULL) ? NULL : (resolved_nodes = (gee_collection_object_unref (resolved_nodes), NULL));
				}
			}
			(variable == NULL) ? NULL : (variable = (g_object_unref (variable), NULL));
		}
		(_variable_it == NULL) ? NULL : (_variable_it = (gee_collection_object_unref (_variable_it), NULL));
	}
	(target_variable == NULL) ? NULL : (target_variable = (g_object_unref (target_variable), NULL));
	name = (g_free (name), NULL);
}


static char* vbf_am_project_manager_process_include_directives (VbfAmProjectManager* self, const char* makefile, const char* buffer, GError** error) {
	GError * inner_error;
	const char* _tmp0;
	char* res;
	char** _tmp2;
	gint lines_size;
	gint lines_length1;
	char** _tmp1;
	char** lines;
	char* _tmp13;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (makefile != NULL, NULL);
	g_return_val_if_fail (buffer != NULL, NULL);
	inner_error = NULL;
	_tmp0 = NULL;
	res = (_tmp0 = buffer, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	_tmp2 = NULL;
	_tmp1 = NULL;
	lines = (_tmp2 = _tmp1 = g_strsplit (buffer, "\n", 0), lines_length1 = _vala_array_length (_tmp1), lines_size = lines_length1, _tmp2);
	{
		char** line_collection;
		int line_collection_length1;
		int line_it;
		line_collection = lines;
		line_collection_length1 = lines_length1;
		for (line_it = 0; line_it < lines_length1; line_it = line_it + 1) {
			const char* _tmp12;
			char* line;
			_tmp12 = NULL;
			line = (_tmp12 = line_collection[line_it], (_tmp12 == NULL) ? NULL : g_strdup (_tmp12));
			{
				char* _tmp3;
				char** _tmp5;
				gint tmps_size;
				gint tmps_length1;
				char** _tmp4;
				char** tmps;
				if (_vala_strcmp0 (line, "") == 0) {
					line = (g_free (line), NULL);
					continue;
				}
				_tmp3 = NULL;
				line = (_tmp3 = vbf_am_project_manager_normalize_string (self, line), line = (g_free (line), NULL), _tmp3);
				_tmp5 = NULL;
				_tmp4 = NULL;
				tmps = (_tmp5 = _tmp4 = g_strsplit (line, " ", 2), tmps_length1 = _vala_array_length (_tmp4), tmps_size = tmps_length1, _tmp5);
				if (_vala_strcmp0 (tmps[0], "include") == 0) {
					char* filename;
					char* include_filename;
					char* included_file;
					char* _tmp8;
					gboolean _tmp7;
					char* _tmp6;
					gboolean _tmp9;
					filename = vbf_am_project_manager_normalize_string (self, tmps[1]);
					include_filename = string_replace (makefile, "Makefile.am", filename);
					included_file = NULL;
					_tmp8 = NULL;
					_tmp6 = NULL;
					_tmp9 = (_tmp7 = g_file_get_contents (include_filename, &_tmp6, NULL, &inner_error), included_file = (_tmp8 = _tmp6, included_file = (g_free (included_file), NULL), _tmp8), _tmp7);
					if (inner_error != NULL) {
						g_propagate_error (error, inner_error);
						filename = (g_free (filename), NULL);
						include_filename = (g_free (include_filename), NULL);
						included_file = (g_free (included_file), NULL);
						line = (g_free (line), NULL);
						tmps = (_vala_array_free (tmps, tmps_length1, (GDestroyNotify) g_free), NULL);
						res = (g_free (res), NULL);
						lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL);
						return NULL;
					}
					if (_tmp9) {
						char* _tmp11;
						char* _tmp10;
						_tmp11 = NULL;
						_tmp10 = NULL;
						res = (_tmp11 = string_replace (res, _tmp10 = g_strdup_printf ("include %s", filename), included_file), res = (g_free (res), NULL), _tmp11);
						_tmp10 = (g_free (_tmp10), NULL);
					}
					filename = (g_free (filename), NULL);
					include_filename = (g_free (include_filename), NULL);
					included_file = (g_free (included_file), NULL);
				}
				line = (g_free (line), NULL);
				tmps = (_vala_array_free (tmps, tmps_length1, (GDestroyNotify) g_free), NULL);
			}
		}
	}
	_tmp13 = NULL;
	return (_tmp13 = res, lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL), _tmp13);
}


static void vbf_am_project_manager_parse_makefile (VbfAmProjectManager* self, VbfProject* project, const char* project_file, const char* makefile, GError** error) {
	GError * inner_error;
	char* buffer;
	char* _tmp0;
	char* _tmp1;
	char* file;
	char* _tmp4;
	gboolean _tmp3;
	char* _tmp2;
	gboolean _tmp5;
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	g_return_if_fail (project_file != NULL);
	g_return_if_fail (makefile != NULL);
	inner_error = NULL;
	if (!g_str_has_suffix (makefile, "Makefile")) {
		return;
	}
	buffer = NULL;
	_tmp0 = NULL;
	_tmp1 = NULL;
	file = (_tmp1 = g_strconcat (_tmp0 = g_build_filename (project_file, makefile, NULL), ".am", NULL), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
	_tmp4 = NULL;
	_tmp2 = NULL;
	_tmp5 = (_tmp3 = g_file_get_contents (file, &_tmp2, NULL, &inner_error), buffer = (_tmp4 = _tmp2, buffer = (g_free (buffer), NULL), _tmp4), _tmp3);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		buffer = (g_free (buffer), NULL);
		file = (g_free (file), NULL);
		return;
	}
	if (_tmp5) {
		char* _tmp6;
		VbfGroup* _tmp7;
		VbfGroup* group;
		char* _tmp8;
		char* _tmp9;
		GeeList* _tmp10;
		_tmp6 = NULL;
		_tmp7 = NULL;
		group = (_tmp7 = vbf_group_new (project, _tmp6 = string_replace (file, "Makefile.am", "")), _tmp6 = (g_free (_tmp6), NULL), _tmp7);
		vbf_project_add_group (project, group);
		_tmp8 = vbf_am_project_manager_process_include_directives (self, file, buffer, &inner_error);
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
			buffer = (g_free (buffer), NULL);
			file = (g_free (file), NULL);
			return;
		}
		_tmp9 = NULL;
		buffer = (_tmp9 = _tmp8, buffer = (g_free (buffer), NULL), _tmp9);
		vbf_am_project_manager_parse_variables (self, project, buffer, group, &inner_error);
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
			buffer = (g_free (buffer), NULL);
			file = (g_free (file), NULL);
			return;
		}
		_tmp10 = NULL;
		vbf_am_project_manager_resolve_variables (self, project, _tmp10 = vbf_group_get_variables (group));
		(_tmp10 == NULL) ? NULL : (_tmp10 = (gee_collection_object_unref (_tmp10), NULL));
		vbf_am_project_manager_parse_targets (self, group);
		vbf_am_project_manager_parse_group_extended_info (self, group, buffer);
		(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
	}
	buffer = (g_free (buffer), NULL);
	file = (g_free (file), NULL);
}


static void vbf_am_project_manager_parse_group_extended_info (VbfAmProjectManager* self, VbfGroup* group, const char* buffer) {
	char** _tmp1;
	gint lines_size;
	gint lines_length1;
	char** _tmp0;
	char** lines;
	VbfTarget* target;
	g_return_if_fail (self != NULL);
	g_return_if_fail (group != NULL);
	g_return_if_fail (buffer != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	lines = (_tmp1 = _tmp0 = g_strsplit (buffer, "\n", 0), lines_length1 = _vala_array_length (_tmp0), lines_size = lines_length1, _tmp1);
	target = NULL;
	{
		char** line_collection;
		int line_collection_length1;
		int line_it;
		line_collection = lines;
		line_collection_length1 = lines_length1;
		for (line_it = 0; line_it < lines_length1; line_it = line_it + 1) {
			const char* _tmp20;
			char* line;
			_tmp20 = NULL;
			line = (_tmp20 = line_collection[line_it], (_tmp20 == NULL) ? NULL : g_strdup (_tmp20));
			{
				char* _tmp2;
				char** _tmp11;
				gint tmps_size;
				gint tmps_length1;
				char** _tmp10;
				char** tmps;
				gint count;
				_tmp2 = NULL;
				line = (_tmp2 = vbf_am_project_manager_normalize_string (self, line), line = (g_free (line), NULL), _tmp2);
				if (_vala_strcmp0 (line, "") == 0) {
					if (target != NULL) {
						VbfTarget* _tmp3;
						_tmp3 = NULL;
						target = (_tmp3 = NULL, (target == NULL) ? NULL : (target = (g_object_unref (target), NULL)), _tmp3);
					}
					line = (g_free (line), NULL);
					continue;
				}
				/*skip this line
				try extract the the rule target name*/
				if (target == NULL) {
					char** _tmp5;
					gint rule_parts_size;
					gint rule_parts_length1;
					char** _tmp4;
					char** rule_parts;
					_tmp5 = NULL;
					_tmp4 = NULL;
					rule_parts = (_tmp5 = _tmp4 = g_strsplit (line, ":", 2), rule_parts_length1 = _vala_array_length (_tmp4), rule_parts_size = rule_parts_length1, _tmp5);
					if (rule_parts_length1 == 2) {
						char** _tmp7;
						gint trgs_size;
						gint trgs_length1;
						char** _tmp6;
						char** trgs;
						_tmp7 = NULL;
						_tmp6 = NULL;
						trgs = (_tmp7 = _tmp6 = g_strsplit (rule_parts[0], " ", 0), trgs_length1 = _vala_array_length (_tmp6), trgs_size = trgs_length1, _tmp7);
						/*TODO: add support for multitarget rules if required*/
						{
							char** trg_collection;
							int trg_collection_length1;
							int trg_it;
							trg_collection = trgs;
							trg_collection_length1 = trgs_length1;
							for (trg_it = 0; trg_it < trgs_length1; trg_it = trg_it + 1) {
								const char* _tmp9;
								char* trg;
								_tmp9 = NULL;
								trg = (_tmp9 = trg_collection[trg_it], (_tmp9 == NULL) ? NULL : g_strdup (_tmp9));
								{
									VbfTarget* _tmp8;
									_tmp8 = NULL;
									target = (_tmp8 = vbf_group_get_target_for_id (group, trg), (target == NULL) ? NULL : (target = (g_object_unref (target), NULL)), _tmp8);
									if (target != NULL) {
										trg = (g_free (trg), NULL);
										break;
									}
									trg = (g_free (trg), NULL);
								}
							}
						}
						trgs = (_vala_array_free (trgs, trgs_length1, (GDestroyNotify) g_free), NULL);
					}
					rule_parts = (_vala_array_free (rule_parts, rule_parts_length1, (GDestroyNotify) g_free), NULL);
				}
				_tmp11 = NULL;
				_tmp10 = NULL;
				tmps = (_tmp11 = _tmp10 = g_strsplit (line, " ", 0), tmps_length1 = _vala_array_length (_tmp10), tmps_size = tmps_length1, _tmp11);
				count = tmps_length1;
				{
					gint idx;
					idx = 0;
					for (; idx < count; idx++) {
						gboolean _tmp12;
						_tmp12 = FALSE;
						if (_vala_strcmp0 (tmps[idx], "--vapidir") == 0) {
							_tmp12 = (idx + 1) < count;
						} else {
							_tmp12 = FALSE;
						}
						if (_tmp12) {
							const char* _tmp13;
							char* tmp;
							_tmp13 = NULL;
							tmp = (_tmp13 = tmps[idx + 1], (_tmp13 == NULL) ? NULL : g_strdup (_tmp13));
							if (g_str_has_prefix (tmp, ".")) {
								char* _tmp14;
								_tmp14 = NULL;
								tmp = (_tmp14 = g_build_filename (group->project->id, group->name, tmp, NULL), tmp = (g_free (tmp), NULL), _tmp14);
							}
							if (target != NULL) {
								vbf_target_add_include_dir (target, tmp);
							} else {
								vbf_group_add_include_dir (group, tmp);
							}
							idx++;
							tmp = (g_free (tmp), NULL);
						} else {
							gboolean _tmp15;
							_tmp15 = FALSE;
							if (_vala_strcmp0 (tmps[idx], "--pkg") == 0) {
								_tmp15 = (idx + 1) < count;
							} else {
								_tmp15 = FALSE;
							}
							if (_tmp15) {
								const char* _tmp16;
								char* tmp;
								_tmp16 = NULL;
								tmp = (_tmp16 = tmps[idx + 1], (_tmp16 == NULL) ? NULL : g_strdup (_tmp16));
								if (target != NULL) {
									VbfPackage* _tmp17;
									_tmp17 = NULL;
									vbf_target_add_package (target, _tmp17 = vbf_package_new (tmp));
									(_tmp17 == NULL) ? NULL : (_tmp17 = (g_object_unref (_tmp17), NULL));
								} else {
									VbfPackage* _tmp18;
									_tmp18 = NULL;
									vbf_group_add_package (group, _tmp18 = vbf_package_new (tmp));
									(_tmp18 == NULL) ? NULL : (_tmp18 = (g_object_unref (_tmp18), NULL));
								}
								idx++;
								tmp = (g_free (tmp), NULL);
							} else {
								if (_vala_strcmp0 (tmps[idx], "--library") == 0) {
									const char* _tmp19;
									char* tmp;
									_tmp19 = NULL;
									tmp = (_tmp19 = tmps[idx + 1], (_tmp19 == NULL) ? NULL : g_strdup (_tmp19));
									if (target != NULL) {
										vbf_target_add_built_library (target, tmp);
									} else {
										vbf_group_add_built_library (group, tmp);
									}
									idx++;
									tmp = (g_free (tmp), NULL);
								}
							}
						}
					}
				}
				line = (g_free (line), NULL);
				tmps = (_vala_array_free (tmps, tmps_length1, (GDestroyNotify) g_free), NULL);
			}
		}
	}
	lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL);
	(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
}


static char* vbf_am_project_manager_normalize_string (VbfAmProjectManager* self, const char* data) {
	char* res;
	char* _tmp1;
	char* old;
	char* _tmp6;
	const char* _tmp5;
	char* _tmp9;
	g_return_val_if_fail (self != NULL, NULL);
	if (data == NULL) {
		return NULL;
	}
	res = string_replace (data, "\n", " ");
	_tmp1 = NULL;
	res = (_tmp1 = string_replace (res, "\t", " "), res = (g_free (res), NULL), _tmp1);
	old = NULL;
	while (_vala_strcmp0 (old, res) != 0) {
		char* _tmp3;
		const char* _tmp2;
		char* _tmp4;
		_tmp3 = NULL;
		_tmp2 = NULL;
		old = (_tmp3 = (_tmp2 = res, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), old = (g_free (old), NULL), _tmp3);
		_tmp4 = NULL;
		res = (_tmp4 = string_replace (res, "  ", " "), res = (g_free (res), NULL), _tmp4);
	}
	_tmp6 = NULL;
	_tmp5 = NULL;
	res = (_tmp6 = (_tmp5 = g_strstrip (res), (_tmp5 == NULL) ? NULL : g_strdup (_tmp5)), res = (g_free (res), NULL), _tmp6);
	if (g_str_has_prefix (res, "[")) {
		char* _tmp7;
		_tmp7 = NULL;
		res = (_tmp7 = string_substring (res, (glong) 1, string_get_length (res) - 1), res = (g_free (res), NULL), _tmp7);
	}
	if (g_str_has_suffix (res, "]")) {
		char* _tmp8;
		_tmp8 = NULL;
		res = (_tmp8 = string_substring (res, (glong) 0, string_get_length (res) - 1), res = (g_free (res), NULL), _tmp8);
	}
	_tmp9 = NULL;
	return (_tmp9 = res, old = (g_free (old), NULL), _tmp9);
}


VbfAmProjectManager* vbf_am_project_manager_construct (GType object_type) {
	VbfAmProjectManager * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VbfAmProjectManager* vbf_am_project_manager_new (void) {
	return vbf_am_project_manager_construct (VBF_AM_TYPE_PROJECT_MANAGER);
}


static void vbf_am_project_manager_class_init (VbfAmProjectManagerClass * klass) {
	vbf_am_project_manager_parent_class = g_type_class_peek_parent (klass);
}


static void vbf_am_project_manager_vbf_iproject_manager_interface_init (VbfIProjectManagerIface * iface) {
	vbf_am_project_manager_vbf_iproject_manager_parent_iface = g_type_interface_peek_parent (iface);
	iface->probe = vbf_am_project_manager_real_probe;
	iface->open = vbf_am_project_manager_real_open;
	iface->refresh = vbf_am_project_manager_real_refresh;
}


static void vbf_am_project_manager_instance_init (VbfAmProjectManager * self) {
}


GType vbf_am_project_manager_get_type (void) {
	static GType vbf_am_project_manager_type_id = 0;
	if (vbf_am_project_manager_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VbfAmProjectManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vbf_am_project_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VbfAmProjectManager), 0, (GInstanceInitFunc) vbf_am_project_manager_instance_init, NULL };
		static const GInterfaceInfo vbf_iproject_manager_info = { (GInterfaceInitFunc) vbf_am_project_manager_vbf_iproject_manager_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		vbf_am_project_manager_type_id = g_type_register_static (G_TYPE_OBJECT, "VbfAmProjectManager", &g_define_type_info, 0);
		g_type_add_interface_static (vbf_am_project_manager_type_id, VBF_TYPE_IPROJECT_MANAGER, &vbf_iproject_manager_info);
	}
	return vbf_am_project_manager_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




