/*
 *  vbfproject.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vbfproject.h>
#include <gee/arraylist.h>
#include <gio/gio.h>
#include <gee/readonlylist.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <gee/collection.h>
#include <vbfgroup.h>
#include <vbfmodule.h>
#include <vbfiprojectmanager.h>




struct _VbfProjectPrivate {
	GeeList* groups;
	GeeList* modules;
	GeeList* variables;
	GeeList* file_mons;
	gboolean in_refresh;
};

#define VBF_PROJECT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VBF_TYPE_PROJECT, VbfProjectPrivate))
enum  {
	VBF_PROJECT_DUMMY_PROPERTY
};
static char* vbf_project_real_to_string (VbfConfigNode* base);
static void _vbf_project_on_project_file_changed_g_file_monitor_changed (GFileMonitor* _sender, GFile* file, GFile* other_file, GFileMonitorEvent event_type, gpointer self);
static void vbf_project_on_project_file_changed (VbfProject* self, GFileMonitor* sender, GFile* file, GFile* other_file, GFileMonitorEvent event_type);
static gpointer vbf_project_parent_class = NULL;
static void vbf_project_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



VbfProject* vbf_project_construct (GType object_type, const char* id) {
	VbfProject * self;
	char* _tmp1;
	const char* _tmp0;
	g_return_val_if_fail (id != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->id = (_tmp1 = (_tmp0 = id, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->id = (g_free (self->id), NULL), _tmp1);
	return self;
}


VbfProject* vbf_project_new (const char* id) {
	return vbf_project_construct (VBF_TYPE_PROJECT, id);
}


GeeList* vbf_project_get_groups (VbfProject* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (VBF_TYPE_GROUP, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->groups);
}


VbfGroup* vbf_project_get_group (VbfProject* self, const char* id) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (id != NULL, NULL);
	{
		GeeIterator* _group_it;
		_group_it = gee_iterable_iterator ((GeeIterable*) self->priv->groups);
		while (gee_iterator_next (_group_it)) {
			VbfGroup* group;
			group = (VbfGroup*) gee_iterator_get (_group_it);
			if (_vala_strcmp0 (group->id, id) == 0) {
				VbfGroup* _tmp0;
				_tmp0 = NULL;
				return (_tmp0 = group, (_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL)), _tmp0);
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	return NULL;
}


void vbf_project_add_group (VbfProject* self, VbfGroup* group) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (group != NULL);
	gee_collection_add ((GeeCollection*) self->priv->groups, group);
}


GeeList* vbf_project_get_modules (VbfProject* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (VBF_TYPE_MODULE, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->modules);
}


void vbf_project_add_module (VbfProject* self, VbfModule* module) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (module != NULL);
	gee_collection_add ((GeeCollection*) self->priv->modules, module);
}


GeeList* vbf_project_get_variables (VbfProject* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (VBF_TYPE_VARIABLE, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->variables);
}


void vbf_project_add_variable (VbfProject* self, VbfVariable* variable) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (variable != NULL);
	gee_collection_add ((GeeCollection*) self->priv->variables, variable);
}


static char* vbf_project_real_to_string (VbfConfigNode* base) {
	VbfProject * self;
	self = (VbfProject*) base;
	return g_strdup_printf ("%s %s: %s", self->name, self->version, self->id);
}


void vbf_project_clear (VbfProject* self) {
	g_return_if_fail (self != NULL);
	gee_collection_clear ((GeeCollection*) self->priv->groups);
	gee_collection_clear ((GeeCollection*) self->priv->variables);
	gee_collection_clear ((GeeCollection*) self->priv->modules);
	vbf_project_cleanup_file_monitors (self);
}


static void _vbf_project_on_project_file_changed_g_file_monitor_changed (GFileMonitor* _sender, GFile* file, GFile* other_file, GFileMonitorEvent event_type, gpointer self) {
	vbf_project_on_project_file_changed (self, _sender, file, other_file, event_type);
}


void vbf_project_setup_file_monitors (VbfProject* self) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	{
		char* fname;
		GFile* file;
		char* _tmp3;
		GFile* _tmp4;
		GFileMonitor* _tmp5;
		GFileMonitor* file_mon;
		fname = NULL;
		file = NULL;
		{
			GeeIterator* _group_it;
			_group_it = gee_iterable_iterator ((GeeIterable*) self->priv->groups);
			while (gee_iterator_next (_group_it)) {
				VbfGroup* group;
				char* _tmp0;
				GFile* _tmp1;
				GFileMonitor* _tmp2;
				GFileMonitor* file_mon;
				group = (VbfGroup*) gee_iterator_get (_group_it);
				_tmp0 = NULL;
				fname = (_tmp0 = g_build_filename (group->id, "Makefile.am", NULL), fname = (g_free (fname), NULL), _tmp0);
				_tmp1 = NULL;
				file = (_tmp1 = g_file_new_for_path (fname), (file == NULL) ? NULL : (file = (g_object_unref (file), NULL)), _tmp1);
				_tmp2 = NULL;
				file_mon = (_tmp2 = g_file_monitor_file (file, G_FILE_MONITOR_NONE, NULL, &inner_error), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
				if (inner_error != NULL) {
					(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
					(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
					fname = (g_free (fname), NULL);
					(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
					goto __catch0_g_error;
					goto __finally0;
				}
				gee_collection_add ((GeeCollection*) self->priv->file_mons, file_mon);
				g_signal_connect_object (file_mon, "changed", (GCallback) _vbf_project_on_project_file_changed_g_file_monitor_changed, self, 0);
				(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
				(file_mon == NULL) ? NULL : (file_mon = (g_object_unref (file_mon), NULL));
			}
			(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
		}
		_tmp3 = NULL;
		fname = (_tmp3 = g_build_filename (self->id, "configure.ac", NULL), fname = (g_free (fname), NULL), _tmp3);
		_tmp4 = NULL;
		file = (_tmp4 = g_file_new_for_path (fname), (file == NULL) ? NULL : (file = (g_object_unref (file), NULL)), _tmp4);
		_tmp5 = NULL;
		file_mon = (_tmp5 = g_file_monitor_file (file, G_FILE_MONITOR_NONE, NULL, &inner_error), (_tmp5 == NULL) ? NULL : g_object_ref (_tmp5));
		if (inner_error != NULL) {
			fname = (g_free (fname), NULL);
			(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
			goto __catch0_g_error;
			goto __finally0;
		}
		gee_collection_add ((GeeCollection*) self->priv->file_mons, file_mon);
		g_signal_connect_object (file_mon, "changed", (GCallback) _vbf_project_on_project_file_changed_g_file_monitor_changed, self, 0);
		fname = (g_free (fname), NULL);
		(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		(file_mon == NULL) ? NULL : (file_mon = (g_object_unref (file_mon), NULL));
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_critical ("vbfproject.vala:122: setup_file_monitors error: %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally0:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


void vbf_project_cleanup_file_monitors (VbfProject* self) {
	g_return_if_fail (self != NULL);
	{
		GeeIterator* _file_mon_it;
		_file_mon_it = gee_iterable_iterator ((GeeIterable*) self->priv->file_mons);
		while (gee_iterator_next (_file_mon_it)) {
			GFileMonitor* file_mon;
			guint _tmp0;
			file_mon = (GFileMonitor*) gee_iterator_get (_file_mon_it);
			g_signal_handlers_disconnect_matched (file_mon, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("changed", G_TYPE_FILE_MONITOR, &_tmp0, NULL, FALSE), _tmp0), 0, NULL, (GCallback) _vbf_project_on_project_file_changed_g_file_monitor_changed, self);
			g_file_monitor_cancel (file_mon);
			(file_mon == NULL) ? NULL : (file_mon = (g_object_unref (file_mon), NULL));
		}
		(_file_mon_it == NULL) ? NULL : (_file_mon_it = (gee_collection_object_unref (_file_mon_it), NULL));
	}
	gee_collection_clear ((GeeCollection*) self->priv->file_mons);
}


static void vbf_project_on_project_file_changed (VbfProject* self, GFileMonitor* sender, GFile* file, GFile* other_file, GFileMonitorEvent event_type) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (file != NULL);
	vbf_project_update (self);
}


void vbf_project_update (VbfProject* self) {
	g_return_if_fail (self != NULL);
	if (self->priv->in_refresh) {
		return;
	}
	self->priv->in_refresh = TRUE;
	vbf_iproject_manager_refresh (self->backend, self);
	g_signal_emit_by_name (self, "updated");
	self->priv->in_refresh = FALSE;
}


static void vbf_project_class_init (VbfProjectClass * klass) {
	vbf_project_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VbfProjectPrivate));
	G_OBJECT_CLASS (klass)->finalize = vbf_project_finalize;
	VBF_CONFIG_NODE_CLASS (klass)->to_string = vbf_project_real_to_string;
	g_signal_new ("updated", VBF_TYPE_PROJECT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void vbf_project_instance_init (VbfProject * self) {
	self->priv = VBF_PROJECT_GET_PRIVATE (self);
	self->priv->groups = (GeeList*) gee_array_list_new (VBF_TYPE_GROUP, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->modules = (GeeList*) gee_array_list_new (VBF_TYPE_MODULE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->variables = (GeeList*) gee_array_list_new (VBF_TYPE_VARIABLE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->file_mons = (GeeList*) gee_array_list_new (G_TYPE_FILE_MONITOR, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->in_refresh = FALSE;
	self->backend = NULL;
}


static void vbf_project_finalize (GObject* obj) {
	VbfProject * self;
	self = VBF_PROJECT (obj);
	self->id = (g_free (self->id), NULL);
	self->name = (g_free (self->name), NULL);
	self->url = (g_free (self->url), NULL);
	self->version = (g_free (self->version), NULL);
	self->working_dir = (g_free (self->working_dir), NULL);
	(self->priv->groups == NULL) ? NULL : (self->priv->groups = (gee_collection_object_unref (self->priv->groups), NULL));
	(self->priv->modules == NULL) ? NULL : (self->priv->modules = (gee_collection_object_unref (self->priv->modules), NULL));
	(self->priv->variables == NULL) ? NULL : (self->priv->variables = (gee_collection_object_unref (self->priv->variables), NULL));
	(self->priv->file_mons == NULL) ? NULL : (self->priv->file_mons = (gee_collection_object_unref (self->priv->file_mons), NULL));
	(self->backend == NULL) ? NULL : (self->backend = (g_object_unref (self->backend), NULL));
	G_OBJECT_CLASS (vbf_project_parent_class)->finalize (obj);
}


GType vbf_project_get_type (void) {
	static GType vbf_project_type_id = 0;
	if (vbf_project_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VbfProjectClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vbf_project_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VbfProject), 0, (GInstanceInitFunc) vbf_project_instance_init, NULL };
		vbf_project_type_id = g_type_register_static (VBF_TYPE_CONFIG_NODE, "VbfProject", &g_define_type_info, 0);
	}
	return vbf_project_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




