/*
 *  vbftarget.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vbftarget.h>
#include <gee/arraylist.h>
#include <gee/readonlylist.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <gee/collection.h>
#include <vbfsource.h>
#include <vbfgroup.h>




struct _VbfTargetPrivate {
	GeeList* sources;
	GeeList* files;
	GeeList* packages;
	GeeList* include_dirs;
	GeeList* built_libraries;
};

#define VBF_TARGET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VBF_TYPE_TARGET, VbfTargetPrivate))
enum  {
	VBF_TARGET_DUMMY_PROPERTY
};
static gpointer vbf_target_parent_class = NULL;
static void vbf_target_finalize (GObject* obj);




GType vbf_target_types_get_type (void) {
	static GType vbf_target_types_type_id = 0;
	if (G_UNLIKELY (vbf_target_types_type_id == 0)) {
		static const GEnumValue values[] = {{VBF_TARGET_TYPES_PROGRAM, "VBF_TARGET_TYPES_PROGRAM", "program"}, {VBF_TARGET_TYPES_LIBRARY, "VBF_TARGET_TYPES_LIBRARY", "library"}, {VBF_TARGET_TYPES_DATA, "VBF_TARGET_TYPES_DATA", "data"}, {VBF_TARGET_TYPES_BUILT_SOURCES, "VBF_TARGET_TYPES_BUILT_SOURCES", "built-sources"}, {0, NULL, NULL}};
		vbf_target_types_type_id = g_enum_register_static ("VbfTargetTypes", values);
	}
	return vbf_target_types_type_id;
}


VbfTarget* vbf_target_construct (GType object_type, VbfGroup* group, VbfTargetTypes type, const char* id) {
	VbfTarget * self;
	char* _tmp1;
	const char* _tmp0;
	char* _tmp3;
	const char* _tmp2;
	g_return_val_if_fail (group != NULL, NULL);
	g_return_val_if_fail (id != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	self->group = group;
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->id = (_tmp1 = (_tmp0 = id, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->id = (g_free (self->id), NULL), _tmp1);
	_tmp3 = NULL;
	_tmp2 = NULL;
	self->name = (_tmp3 = (_tmp2 = id, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), self->name = (g_free (self->name), NULL), _tmp3);
	self->type = type;
	return self;
}


VbfTarget* vbf_target_new (VbfGroup* group, VbfTargetTypes type, const char* id) {
	return vbf_target_construct (VBF_TYPE_TARGET, group, type, id);
}


GeeList* vbf_target_get_sources (VbfTarget* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (VBF_TYPE_SOURCE, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->sources);
}


gboolean vbf_target_has_sources_of_type (VbfTarget* self, VbfFileTypes type) {
	g_return_val_if_fail (self != NULL, FALSE);
	{
		GeeIterator* _source_it;
		_source_it = gee_iterable_iterator ((GeeIterable*) self->priv->sources);
		while (gee_iterator_next (_source_it)) {
			VbfSource* source;
			source = (VbfSource*) gee_iterator_get (_source_it);
			if (((VbfFile*) source)->type == type) {
				gboolean _tmp0;
				return (_tmp0 = TRUE, (source == NULL) ? NULL : (source = (g_object_unref (source), NULL)), (_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL)), _tmp0);
			}
			(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
		}
		(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
	}
	return FALSE;
}


void vbf_target_add_source (VbfTarget* self, VbfSource* source) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (source != NULL);
	gee_collection_add ((GeeCollection*) self->priv->sources, source);
}


gboolean vbf_target_has_file_of_type (VbfTarget* self, VbfFileTypes type) {
	g_return_val_if_fail (self != NULL, FALSE);
	{
		GeeIterator* _file_it;
		_file_it = gee_iterable_iterator ((GeeIterable*) self->priv->files);
		while (gee_iterator_next (_file_it)) {
			VbfFile* file;
			file = (VbfFile*) gee_iterator_get (_file_it);
			if (file->type == type) {
				gboolean _tmp0;
				return (_tmp0 = TRUE, (file == NULL) ? NULL : (file = (g_object_unref (file), NULL)), (_file_it == NULL) ? NULL : (_file_it = (gee_collection_object_unref (_file_it), NULL)), _tmp0);
			}
			(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		}
		(_file_it == NULL) ? NULL : (_file_it = (gee_collection_object_unref (_file_it), NULL));
	}
	return FALSE;
}


GeeList* vbf_target_get_files (VbfTarget* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (VBF_TYPE_FILE, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->files);
}


void vbf_target_add_file (VbfTarget* self, VbfFile* file) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (file != NULL);
	gee_collection_add ((GeeCollection*) self->priv->files, file);
}


GeeList* vbf_target_get_packages (VbfTarget* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (VBF_TYPE_PACKAGE, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->packages);
}


void vbf_target_add_package (VbfTarget* self, VbfPackage* package) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (package != NULL);
	gee_collection_add ((GeeCollection*) self->priv->packages, package);
}


GeeList* vbf_target_get_include_dirs (VbfTarget* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, self->priv->include_dirs);
}


void vbf_target_add_include_dir (VbfTarget* self, const char* dir) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (dir != NULL);
	gee_collection_add ((GeeCollection*) self->priv->include_dirs, dir);
}


GeeList* vbf_target_get_built_libraries (VbfTarget* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, self->priv->built_libraries);
}


void vbf_target_add_built_library (VbfTarget* self, const char* dir) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (dir != NULL);
	gee_collection_add ((GeeCollection*) self->priv->built_libraries, dir);
}


static void vbf_target_class_init (VbfTargetClass * klass) {
	vbf_target_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VbfTargetPrivate));
	G_OBJECT_CLASS (klass)->finalize = vbf_target_finalize;
}


static void vbf_target_instance_init (VbfTarget * self) {
	self->priv = VBF_TARGET_GET_PRIVATE (self);
	self->no_install = FALSE;
	self->priv->sources = (GeeList*) gee_array_list_new (VBF_TYPE_SOURCE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->files = (GeeList*) gee_array_list_new (VBF_TYPE_FILE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->packages = (GeeList*) gee_array_list_new (VBF_TYPE_PACKAGE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->include_dirs = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
	self->priv->built_libraries = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
}


static void vbf_target_finalize (GObject* obj) {
	VbfTarget * self;
	self = VBF_TARGET (obj);
	self->id = (g_free (self->id), NULL);
	self->name = (g_free (self->name), NULL);
	(self->priv->sources == NULL) ? NULL : (self->priv->sources = (gee_collection_object_unref (self->priv->sources), NULL));
	(self->priv->files == NULL) ? NULL : (self->priv->files = (gee_collection_object_unref (self->priv->files), NULL));
	(self->priv->packages == NULL) ? NULL : (self->priv->packages = (gee_collection_object_unref (self->priv->packages), NULL));
	(self->priv->include_dirs == NULL) ? NULL : (self->priv->include_dirs = (gee_collection_object_unref (self->priv->include_dirs), NULL));
	(self->priv->built_libraries == NULL) ? NULL : (self->priv->built_libraries = (gee_collection_object_unref (self->priv->built_libraries), NULL));
	G_OBJECT_CLASS (vbf_target_parent_class)->finalize (obj);
}


GType vbf_target_get_type (void) {
	static GType vbf_target_type_id = 0;
	if (vbf_target_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VbfTargetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vbf_target_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VbfTarget), 0, (GInstanceInitFunc) vbf_target_instance_init, NULL };
		vbf_target_type_id = g_type_register_static (G_TYPE_OBJECT, "VbfTarget", &g_define_type_info, 0);
	}
	return vbf_target_type_id;
}




