/*
 *  vtgplugininstance.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgplugininstance.h"
#include <gee/arraylist.h>
#include <gee/list.h>
#include <gee/collection.h>
#include <gedit/gedit-encodings.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <gtk/gtk.h>
#include <gtksourceview/gtksourceview.h>
#include <vscsymbolcompletion.h>
#include <gedit/gedit-app.h>
#include "vtgplugin.h"
#include "vtgsymbolcompletionhelper.h"
#include "vtgoutputview.h"
#include "vtgprojectmanagerui.h"
#include "vtgconfiguration.h"




struct _VtgPluginInstancePrivate {
	GeditWindow* _window;
	VtgProjectManagerUi* _project_manager_ui;
	VtgOutputView* _output_view;
	GeeList* _scs;
	GeeList* _bcs;
};

#define VTG_PLUGIN_INSTANCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstancePrivate))
enum  {
	VTG_PLUGIN_INSTANCE_DUMMY_PROPERTY,
	VTG_PLUGIN_INSTANCE_OUTPUT_VIEW,
	VTG_PLUGIN_INSTANCE_PROJECT_MANAGER_UI,
	VTG_PLUGIN_INSTANCE_WINDOW
};
static void vtg_plugin_instance_on_tab_added (GeditWindow* sender, GeditTab* tab, VtgPluginInstance* instance);
static void vtg_plugin_instance_on_tab_removed (GeditWindow* sender, GeditTab* tab, VtgPluginInstance* instance);
static void vtg_plugin_instance_on_notify_language (GeditDocument* sender, GParamSpec* pspec, VtgPluginInstance* instance);
static gpointer vtg_plugin_instance_parent_class = NULL;
static void vtg_plugin_instance_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



VtgPluginInstance* vtg_plugin_instance_construct (GType object_type, VtgPlugin* plugin, GeditWindow* window) {
	VtgPluginInstance * self;
	VtgOutputView* _tmp1;
	VtgProjectManagerUi* _tmp2;
	g_return_val_if_fail (plugin != NULL, NULL);
	g_return_val_if_fail (window != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	self->plugin = plugin;
	self->priv->_window = window;
	g_signal_connect_after (self->priv->_window, "tab-added", (GCallback) vtg_plugin_instance_on_tab_added, self);
	g_signal_connect_after (self->priv->_window, "tab-removed", (GCallback) vtg_plugin_instance_on_tab_removed, self);
	vtg_plugin_instance_initialize_views (self);
	{
		GList* doc_collection;
		GList* doc_it;
		doc_collection = gedit_window_get_documents (self->priv->_window);
		for (doc_it = doc_collection; doc_it != NULL; doc_it = doc_it->next) {
			GeditDocument* _tmp0;
			GeditDocument* doc;
			_tmp0 = NULL;
			doc = (_tmp0 = (GeditDocument*) doc_it->data, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
			{
				vtg_plugin_instance_initialize_document (self, doc);
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
			}
		}
	}
	_tmp1 = NULL;
	self->priv->_output_view = (_tmp1 = vtg_output_view_new (self), (self->priv->_output_view == NULL) ? NULL : (self->priv->_output_view = (g_object_unref (self->priv->_output_view), NULL)), _tmp1);
	_tmp2 = NULL;
	self->priv->_project_manager_ui = (_tmp2 = vtg_project_manager_ui_new (self), (self->priv->_project_manager_ui == NULL) ? NULL : (self->priv->_project_manager_ui = (g_object_unref (self->priv->_project_manager_ui), NULL)), _tmp2);
	return self;
}


VtgPluginInstance* vtg_plugin_instance_new (VtgPlugin* plugin, GeditWindow* window) {
	return vtg_plugin_instance_construct (VTG_TYPE_PLUGIN_INSTANCE, plugin, window);
}


static void vtg_plugin_instance_on_tab_added (GeditWindow* sender, GeditTab* tab, VtgPluginInstance* instance) {
	GeditDocument* _tmp0;
	GeditDocument* doc;
	VtgProjectDescriptor* project;
	gboolean _tmp1;
	g_return_if_fail (sender != NULL);
	g_return_if_fail (tab != NULL);
	g_return_if_fail (instance != NULL);
	_tmp0 = NULL;
	doc = (_tmp0 = gedit_tab_get_document (tab), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	project = vtg_plugin_project_descriptor_find_from_document (instance->plugin, doc);
	_tmp1 = FALSE;
	if (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc) != NULL) {
		_tmp1 = _vala_strcmp0 (gtk_source_language_get_id (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc)), "vala") == 0;
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		GeditView* _tmp2;
		GeditView* view;
		_tmp2 = NULL;
		view = (_tmp2 = gedit_tab_get_view (tab), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
		vtg_plugin_instance_initialize_view (instance, project, view);
		(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
	}
	vtg_plugin_instance_initialize_document (instance, doc);
	(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
	(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
}


static void vtg_plugin_instance_on_tab_removed (GeditWindow* sender, GeditTab* tab, VtgPluginInstance* instance) {
	GeditView* _tmp0;
	GeditView* view;
	GeditDocument* _tmp1;
	GeditDocument* doc;
	g_return_if_fail (sender != NULL);
	g_return_if_fail (tab != NULL);
	g_return_if_fail (instance != NULL);
	_tmp0 = NULL;
	view = (_tmp0 = gedit_tab_get_view (tab), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	_tmp1 = NULL;
	doc = (_tmp1 = gedit_tab_get_document (tab), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	vtg_plugin_instance_uninitialize_view (instance, view);
	vtg_plugin_instance_uninitialize_document (instance, doc);
	(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
	(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
}


void vtg_plugin_instance_initialize_views (VtgPluginInstance* self) {
	g_return_if_fail (self != NULL);
	{
		GList* view_collection;
		GList* view_it;
		view_collection = gedit_window_get_views (self->priv->_window);
		for (view_it = view_collection; view_it != NULL; view_it = view_it->next) {
			GeditView* _tmp2;
			GeditView* view;
			_tmp2 = NULL;
			view = (_tmp2 = (GeditView*) view_it->data, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
			{
				GeditDocument* _tmp0;
				GeditDocument* doc;
				gboolean _tmp1;
				_tmp0 = NULL;
				doc = (_tmp0 = GEDIT_DOCUMENT (gtk_text_view_get_buffer ((GtkTextView*) view)), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
				_tmp1 = FALSE;
				if (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc) != NULL) {
					_tmp1 = _vala_strcmp0 (gtk_source_language_get_id (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc)), "vala") == 0;
				} else {
					_tmp1 = FALSE;
				}
				if (_tmp1) {
					VtgProjectDescriptor* project;
					project = vtg_plugin_project_descriptor_find_from_document (self->plugin, doc);
					vtg_plugin_instance_initialize_view (self, project, view);
					(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
				}
				(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
			}
		}
	}
}


void vtg_plugin_instance_initialize_view (VtgPluginInstance* self, VtgProjectDescriptor* project, GeditView* view) {
	gboolean _tmp0;
	gboolean _tmp1;
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	g_return_if_fail (view != NULL);
	_tmp0 = FALSE;
	if (vtg_configuration_get_symbol_enabled (vtg_plugin_get_config (self->plugin))) {
		_tmp0 = !vtg_plugin_instance_scs_contains (self, view);
	} else {
		_tmp0 = FALSE;
	}
	if (_tmp0) {
		vtg_plugin_instance_activate_symbol (self, project, view);
	} else {
		g_warning ("vtgplugininstance.vala:117: sc already initialized for view");
	}
	_tmp1 = FALSE;
	if (vtg_configuration_get_bracket_enabled (vtg_plugin_get_config (self->plugin))) {
		_tmp1 = !vtg_plugin_instance_bcs_contains (self, view);
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		vtg_plugin_instance_activate_bracket (self, view);
	} else {
		g_warning ("vtgplugininstance.vala:123: bc already initialized vor view");
	}
}


void vtg_plugin_instance_initialize_document (VtgPluginInstance* self, GeditDocument* doc) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (doc != NULL);
	g_signal_connect (doc, "notify::language", (GCallback) vtg_plugin_instance_on_notify_language, self);
}


void vtg_plugin_instance_uninitialize_view (VtgPluginInstance* self, GeditView* view) {
	VtgSymbolCompletionHelper* sc;
	VtgBracketCompletion* bc;
	g_return_if_fail (self != NULL);
	g_return_if_fail (view != NULL);
	sc = vtg_plugin_instance_scs_find_from_view (self, view);
	if (sc != NULL) {
		vtg_plugin_instance_deactivate_symbol (self, sc);
	}
	bc = vtg_plugin_instance_bcs_find_from_view (self, view);
	if (bc != NULL) {
		vtg_plugin_instance_deactivate_bracket (self, bc);
	}
	(sc == NULL) ? NULL : (sc = (g_object_unref (sc), NULL));
	(bc == NULL) ? NULL : (bc = (g_object_unref (bc), NULL));
}


void vtg_plugin_instance_activate_bracket (VtgPluginInstance* self, GeditView* view) {
	VtgBracketCompletion* bc;
	g_return_if_fail (self != NULL);
	g_return_if_fail (view != NULL);
	bc = vtg_bracket_completion_new (self, view);
	gee_collection_add ((GeeCollection*) self->priv->_bcs, bc);
	(bc == NULL) ? NULL : (bc = (g_object_unref (bc), NULL));
}


void vtg_plugin_instance_deactivate_bracket (VtgPluginInstance* self, VtgBracketCompletion* bc) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (bc != NULL);
	vtg_bracket_completion_deactivate (bc);
	gee_collection_remove ((GeeCollection*) self->priv->_bcs, bc);
}


void vtg_plugin_instance_activate_symbol (VtgPluginInstance* self, VtgProjectDescriptor* project, GeditView* view) {
	VtgSymbolCompletionHelper* sc;
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	g_return_if_fail (view != NULL);
	sc = vtg_symbol_completion_helper_new (self, view, project->completion);
	gee_collection_add ((GeeCollection*) self->priv->_scs, sc);
	(sc == NULL) ? NULL : (sc = (g_object_unref (sc), NULL));
}


void vtg_plugin_instance_deactivate_symbol (VtgPluginInstance* self, VtgSymbolCompletionHelper* sc) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sc != NULL);
	vtg_symbol_completion_helper_deactivate (sc);
	gee_collection_remove ((GeeCollection*) self->priv->_scs, sc);
}


void vtg_plugin_instance_deactivate_symbols (VtgPluginInstance* self) {
	gint size;
	g_return_if_fail (self != NULL);
	size = 0;
	while (TRUE) {
		gboolean _tmp0;
		VtgSymbolCompletionHelper* _tmp1;
		_tmp0 = FALSE;
		if (gee_collection_get_size ((GeeCollection*) self->priv->_scs) > 0) {
			_tmp0 = gee_collection_get_size ((GeeCollection*) self->priv->_scs) != size;
		} else {
			_tmp0 = FALSE;
		}
		if (!_tmp0) {
			break;
		}
		size = gee_collection_get_size ((GeeCollection*) self->priv->_scs);
		_tmp1 = NULL;
		vtg_plugin_instance_deactivate_symbol (self, _tmp1 = (VtgSymbolCompletionHelper*) gee_list_get (self->priv->_scs, 0));
		(_tmp1 == NULL) ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL));
	}
}


void vtg_plugin_instance_deactivate_brackets (VtgPluginInstance* self) {
	gint size;
	g_return_if_fail (self != NULL);
	size = 0;
	while (TRUE) {
		gboolean _tmp0;
		VtgBracketCompletion* _tmp1;
		_tmp0 = FALSE;
		if (gee_collection_get_size ((GeeCollection*) self->priv->_bcs) > 0) {
			_tmp0 = gee_collection_get_size ((GeeCollection*) self->priv->_bcs) != size;
		} else {
			_tmp0 = FALSE;
		}
		if (!_tmp0) {
			break;
		}
		size = gee_collection_get_size ((GeeCollection*) self->priv->_bcs);
		_tmp1 = NULL;
		vtg_plugin_instance_deactivate_bracket (self, _tmp1 = (VtgBracketCompletion*) gee_list_get (self->priv->_bcs, 0));
		(_tmp1 == NULL) ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL));
	}
}


gboolean vtg_plugin_instance_bcs_contains (VtgPluginInstance* self, GeditView* view) {
	VtgBracketCompletion* _tmp0;
	gboolean _tmp1;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (view != NULL, FALSE);
	_tmp0 = NULL;
	return (_tmp1 = (_tmp0 = vtg_plugin_instance_bcs_find_from_view (self, view)) != NULL, (_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)), _tmp1);
}


VtgBracketCompletion* vtg_plugin_instance_bcs_find_from_view (VtgPluginInstance* self, GeditView* view) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (view != NULL, NULL);
	{
		GeeIterator* _bc_it;
		_bc_it = gee_iterable_iterator ((GeeIterable*) self->priv->_bcs);
		while (gee_iterator_next (_bc_it)) {
			VtgBracketCompletion* bc;
			bc = (VtgBracketCompletion*) gee_iterator_get (_bc_it);
			if (vtg_bracket_completion_get_view (bc) == view) {
				VtgBracketCompletion* _tmp0;
				_tmp0 = NULL;
				return (_tmp0 = bc, (_bc_it == NULL) ? NULL : (_bc_it = (gee_collection_object_unref (_bc_it), NULL)), _tmp0);
			}
			(bc == NULL) ? NULL : (bc = (g_object_unref (bc), NULL));
		}
		(_bc_it == NULL) ? NULL : (_bc_it = (gee_collection_object_unref (_bc_it), NULL));
	}
	return NULL;
}


gboolean vtg_plugin_instance_scs_contains (VtgPluginInstance* self, GeditView* view) {
	VtgSymbolCompletionHelper* _tmp0;
	gboolean _tmp1;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (view != NULL, FALSE);
	_tmp0 = NULL;
	return (_tmp1 = (_tmp0 = vtg_plugin_instance_scs_find_from_view (self, view)) != NULL, (_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)), _tmp1);
}


VtgSymbolCompletionHelper* vtg_plugin_instance_scs_find_from_view (VtgPluginInstance* self, GeditView* view) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (view != NULL, NULL);
	{
		GeeIterator* _sc_it;
		_sc_it = gee_iterable_iterator ((GeeIterable*) self->priv->_scs);
		while (gee_iterator_next (_sc_it)) {
			VtgSymbolCompletionHelper* sc;
			sc = (VtgSymbolCompletionHelper*) gee_iterator_get (_sc_it);
			if (vtg_symbol_completion_helper_get_view (sc) == view) {
				VtgSymbolCompletionHelper* _tmp0;
				_tmp0 = NULL;
				return (_tmp0 = sc, (_sc_it == NULL) ? NULL : (_sc_it = (gee_collection_object_unref (_sc_it), NULL)), _tmp0);
			}
			(sc == NULL) ? NULL : (sc = (g_object_unref (sc), NULL));
		}
		(_sc_it == NULL) ? NULL : (_sc_it = (gee_collection_object_unref (_sc_it), NULL));
	}
	return NULL;
}


void vtg_plugin_instance_uninitialize_document (VtgPluginInstance* self, GeditDocument* doc) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (doc != NULL);
	g_signal_handlers_disconnect_by_func (doc, (void*) vtg_plugin_instance_on_notify_language, self);
}


GeditTab* vtg_plugin_instance_activate_uri (VtgPluginInstance* self, const char* uri, gint line, gint col) {
	GeditTab* tab;
	GeditDocument* existing_doc;
	GeditTab* _tmp8;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (uri != NULL, NULL);
	tab = NULL;
	existing_doc = NULL;
	{
		GList* doc_collection;
		GList* doc_it;
		doc_collection = gedit_window_get_documents (self->priv->_window);
		for (doc_it = doc_collection; doc_it != NULL; doc_it = doc_it->next) {
			GeditDocument* _tmp4;
			GeditDocument* doc;
			_tmp4 = NULL;
			doc = (_tmp4 = (GeditDocument*) doc_it->data, (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4));
			{
				if (_vala_strcmp0 (gedit_document_get_uri (doc), uri) == 0) {
					GeditTab* _tmp1;
					GeditTab* _tmp0;
					GeditDocument* _tmp3;
					GeditDocument* _tmp2;
					_tmp1 = NULL;
					_tmp0 = NULL;
					tab = (_tmp1 = (_tmp0 = gedit_tab_get_from_document (doc), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL)), _tmp1);
					_tmp3 = NULL;
					_tmp2 = NULL;
					existing_doc = (_tmp3 = (_tmp2 = doc, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2)), (existing_doc == NULL) ? NULL : (existing_doc = (g_object_unref (existing_doc), NULL)), _tmp3);
					(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
					break;
				}
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
			}
		}
	}
	if (tab == NULL) {
		GeditTab* _tmp6;
		GeditTab* _tmp5;
		_tmp6 = NULL;
		_tmp5 = NULL;
		tab = (_tmp6 = (_tmp5 = gedit_window_create_tab_from_uri (self->priv->_window, uri, gedit_encoding_get_utf8 (), line, TRUE, TRUE), (_tmp5 == NULL) ? NULL : g_object_ref (_tmp5)), (tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL)), _tmp6);
	} else {
		gboolean _tmp7;
		gedit_window_set_active_tab (self->priv->_window, tab);
		_tmp7 = FALSE;
		if (existing_doc != NULL) {
			_tmp7 = line > 0;
		} else {
			_tmp7 = FALSE;
		}
		if (_tmp7) {
			gedit_document_goto_line (existing_doc, line - 1);
			gedit_view_scroll_to_cursor (gedit_tab_get_view (tab));
		}
	}
	_tmp8 = NULL;
	return (_tmp8 = tab, (existing_doc == NULL) ? NULL : (existing_doc = (g_object_unref (existing_doc), NULL)), _tmp8);
}


void vtg_plugin_instance_activate_display_name (VtgPluginInstance* self, const char* display_name, gint line, gint col) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (display_name != NULL);
	{
		GList* doc_collection;
		GList* doc_it;
		doc_collection = gedit_window_get_documents (self->priv->_window);
		for (doc_it = doc_collection; doc_it != NULL; doc_it = doc_it->next) {
			GeditDocument* _tmp1;
			GeditDocument* doc;
			_tmp1 = NULL;
			doc = (_tmp1 = (GeditDocument*) doc_it->data, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
			{
				if (_vala_strcmp0 (gedit_document_get_short_name_for_display (doc), display_name) == 0) {
					GeditTab* _tmp0;
					GeditTab* tab;
					_tmp0 = NULL;
					tab = (_tmp0 = gedit_tab_get_from_document (doc), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
					gedit_window_set_active_tab (self->priv->_window, tab);
					gedit_document_goto_line (doc, line - 1);
					gedit_view_scroll_to_cursor (gedit_tab_get_view (tab));
					(tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL));
				}
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
			}
		}
	}
}


static void vtg_plugin_instance_on_notify_language (GeditDocument* sender, GParamSpec* pspec, VtgPluginInstance* instance) {
	GeditApp* _tmp0;
	GeditApp* app;
	g_return_if_fail (sender != NULL);
	g_return_if_fail (pspec != NULL);
	g_return_if_fail (instance != NULL);
	/*search the view*/
	_tmp0 = NULL;
	app = (_tmp0 = gedit_app_get_default (), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	{
		GList* view_collection;
		GList* view_it;
		view_collection = gedit_app_get_views (app);
		for (view_it = view_collection; view_it != NULL; view_it = view_it->next) {
			GeditView* _tmp2;
			GeditView* view;
			_tmp2 = NULL;
			view = (_tmp2 = (GeditView*) view_it->data, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
			{
				if (gtk_text_view_get_buffer ((GtkTextView*) view) == GTK_TEXT_BUFFER (sender)) {
					gboolean _tmp1;
					_tmp1 = FALSE;
					if (gtk_source_buffer_get_language ((GtkSourceBuffer*) sender) == NULL) {
						_tmp1 = TRUE;
					} else {
						_tmp1 = _vala_strcmp0 (gtk_source_language_get_id (gtk_source_buffer_get_language ((GtkSourceBuffer*) sender)), "vala") != 0;
					}
					if (_tmp1) {
						vtg_plugin_instance_uninitialize_view (instance, view);
					} else {
						VtgProjectDescriptor* project;
						project = vtg_plugin_project_descriptor_find_from_document (instance->plugin, sender);
						vtg_plugin_instance_initialize_view (instance, project, view);
						(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
					}
					(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
					break;
				}
				(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
			}
		}
	}
	(app == NULL) ? NULL : (app = (g_object_unref (app), NULL));
}


VtgOutputView* vtg_plugin_instance_get_output_view (VtgPluginInstance* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_output_view;
}


VtgProjectManagerUi* vtg_plugin_instance_get_project_manager_ui (VtgPluginInstance* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_project_manager_ui;
}


GeditWindow* vtg_plugin_instance_get_window (VtgPluginInstance* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_window;
}


static void vtg_plugin_instance_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgPluginInstance * self;
	gpointer boxed;
	self = VTG_PLUGIN_INSTANCE (object);
	switch (property_id) {
		case VTG_PLUGIN_INSTANCE_OUTPUT_VIEW:
		g_value_set_object (value, vtg_plugin_instance_get_output_view (self));
		break;
		case VTG_PLUGIN_INSTANCE_PROJECT_MANAGER_UI:
		g_value_set_object (value, vtg_plugin_instance_get_project_manager_ui (self));
		break;
		case VTG_PLUGIN_INSTANCE_WINDOW:
		g_value_set_object (value, vtg_plugin_instance_get_window (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_plugin_instance_class_init (VtgPluginInstanceClass * klass) {
	vtg_plugin_instance_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgPluginInstancePrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_plugin_instance_get_property;
	G_OBJECT_CLASS (klass)->finalize = vtg_plugin_instance_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PLUGIN_INSTANCE_OUTPUT_VIEW, g_param_spec_object ("output-view", "output-view", "output-view", VTG_TYPE_OUTPUT_VIEW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PLUGIN_INSTANCE_PROJECT_MANAGER_UI, g_param_spec_object ("project-manager-ui", "project-manager-ui", "project-manager-ui", VTG_TYPE_PROJECT_MANAGER_UI, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PLUGIN_INSTANCE_WINDOW, g_param_spec_object ("window", "window", "window", GEDIT_TYPE_WINDOW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void vtg_plugin_instance_instance_init (VtgPluginInstance * self) {
	self->priv = VTG_PLUGIN_INSTANCE_GET_PRIVATE (self);
	self->priv->_window = NULL;
	self->priv->_project_manager_ui = NULL;
	self->priv->_output_view = NULL;
	self->priv->_scs = (GeeList*) gee_array_list_new (VTG_TYPE_SYMBOL_COMPLETION_HELPER, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->_bcs = (GeeList*) gee_array_list_new (VTG_TYPE_BRACKET_COMPLETION, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
}


static void vtg_plugin_instance_finalize (GObject* obj) {
	VtgPluginInstance * self;
	self = VTG_PLUGIN_INSTANCE (obj);
	{
		VtgProjectManagerUi* _tmp0;
		VtgOutputView* _tmp1;
		_tmp0 = NULL;
		self->priv->_project_manager_ui = (_tmp0 = NULL, (self->priv->_project_manager_ui == NULL) ? NULL : (self->priv->_project_manager_ui = (g_object_unref (self->priv->_project_manager_ui), NULL)), _tmp0);
		_tmp1 = NULL;
		self->priv->_output_view = (_tmp1 = NULL, (self->priv->_output_view == NULL) ? NULL : (self->priv->_output_view = (g_object_unref (self->priv->_output_view), NULL)), _tmp1);
		self->priv->_window = NULL;
	}
	(self->priv->_project_manager_ui == NULL) ? NULL : (self->priv->_project_manager_ui = (g_object_unref (self->priv->_project_manager_ui), NULL));
	(self->priv->_output_view == NULL) ? NULL : (self->priv->_output_view = (g_object_unref (self->priv->_output_view), NULL));
	(self->priv->_scs == NULL) ? NULL : (self->priv->_scs = (gee_collection_object_unref (self->priv->_scs), NULL));
	(self->priv->_bcs == NULL) ? NULL : (self->priv->_bcs = (gee_collection_object_unref (self->priv->_bcs), NULL));
	G_OBJECT_CLASS (vtg_plugin_instance_parent_class)->finalize (obj);
}


static GType vtg_plugin_instance_type_id = 0;
GType vtg_plugin_instance_get_type (void) {
	return vtg_plugin_instance_type_id;
}


GType vtg_plugin_instance_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgPluginInstanceClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_plugin_instance_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgPluginInstance), 0, (GInstanceInitFunc) vtg_plugin_instance_instance_init, NULL };
	vtg_plugin_instance_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgPluginInstance", &g_define_type_info, 0);
	return vtg_plugin_instance_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




