/*
 *  vtgprojectbuilder.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectbuilder.h"
#include <glib/gi18n-lib.h>
#include <sys/wait.h>
#include <gedit/gedit-window.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <vbfproject.h>
#include <glib/gstdio.h>
#include <signal.h>
#include <sys/types.h>
#include "vtgoutputview.h"




static glong string_get_length (const char* self);
struct _VtgProjectBuilderPrivate {
	VtgPluginInstance* _plugin_instance;
	VtgBuildLogView* _build_view;
	guint _child_watch_id;
	gboolean is_bottom_pane_visible;
	gint last_exit_code;
	GPid _child_pid;
};

#define VTG_PROJECT_BUILDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_PROJECT_BUILDER, VtgProjectBuilderPrivate))
enum  {
	VTG_PROJECT_BUILDER_DUMMY_PROPERTY,
	VTG_PROJECT_BUILDER_PLUGIN_INSTANCE,
	VTG_PROJECT_BUILDER_ERROR_PANE,
	VTG_PROJECT_BUILDER_IS_BUILDING
};
#define VTG_PROJECT_BUILDER_MAKE "make"
static void _vtg_project_builder_on_child_watch_gchild_watch_func (GPid pid, gint status, gpointer self);
static void vtg_project_builder_on_child_watch (VtgProjectBuilder* self, GPid pid, gint status);
static void vtg_project_builder_set_plugin_instance (VtgProjectBuilder* self, VtgPluginInstance* value);
static GObject * vtg_project_builder_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vtg_project_builder_parent_class = NULL;
static void vtg_project_builder_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);



static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


VtgProjectBuilder* vtg_project_builder_construct (GType object_type, VtgPluginInstance* plugin_instance) {
	GParameter * __params;
	GParameter * __params_it;
	VtgProjectBuilder * self;
	g_return_val_if_fail (plugin_instance != NULL, NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "plugin-instance";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN_INSTANCE);
	g_value_set_object (&__params_it->value, plugin_instance);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgProjectBuilder* vtg_project_builder_new (VtgPluginInstance* plugin_instance) {
	return vtg_project_builder_construct (VTG_TYPE_PROJECT_BUILDER, plugin_instance);
}


static void _vtg_project_builder_on_child_watch_gchild_watch_func (GPid pid, gint status, gpointer self) {
	vtg_project_builder_on_child_watch (self, pid, status);
}


gboolean vtg_project_builder_compile_file (VtgProjectBuilder* self, const char* filename, const char* params) {
	GError * inner_error;
	char* working_dir;
	gint stdo;
	gint stde;
	VtgOutputView* _tmp1;
	VtgOutputView* log;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (filename != NULL, FALSE);
	inner_error = NULL;
	if (self->priv->_child_watch_id != 0) {
		return FALSE;
	}
	working_dir = g_path_get_dirname (filename);
	stdo = 0;
	stde = 0;
	_tmp1 = NULL;
	log = (_tmp1 = vtg_plugin_instance_get_output_view (self->priv->_plugin_instance), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	{
		char* cmd;
		gint pars_size;
		gint pars_length1;
		char** pars;
		char* start_message;
		char* _tmp5;
		char* _tmp4;
		char* _tmp6;
		gboolean _tmp8;
		cmd = NULL;
		if (params != NULL) {
			char* _tmp2;
			_tmp2 = NULL;
			cmd = (_tmp2 = g_strdup_printf ("%s %s %s", "valac", params, filename), cmd = (g_free (cmd), NULL), _tmp2);
		} else {
			char* _tmp3;
			_tmp3 = NULL;
			cmd = (_tmp3 = g_strdup_printf ("valac %s", filename), cmd = (g_free (cmd), NULL), _tmp3);
		}
		pars = (pars_length1 = 0, NULL);
		g_shell_parse_argv (cmd, &pars_length1, &pars, &inner_error);
		if (inner_error != NULL) {
			cmd = (g_free (cmd), NULL);
			pars = (_vala_array_free (pars, pars_length1, (GDestroyNotify) g_free), NULL);
			if (inner_error->domain == G_SPAWN_ERROR) {
				goto __catch11_g_spawn_error;
			}
			goto __finally11;
		}
		vtg_output_view_clean_output (log);
		start_message = g_strdup_printf (_ ("Start compiling file: %s\n"), filename);
		vtg_output_view_log_message (log, start_message);
		_tmp5 = NULL;
		_tmp4 = NULL;
		vtg_output_view_log_message (log, _tmp5 = g_strdup_printf ("%s\n\n", _tmp4 = g_strnfill ((gulong) (string_get_length (start_message) - 1), '-')));
		_tmp5 = (g_free (_tmp5), NULL);
		_tmp4 = (g_free (_tmp4), NULL);
		_tmp6 = NULL;
		vtg_output_view_log_message (log, _tmp6 = g_strdup_printf ("%s %s\n", cmd, filename));
		_tmp6 = (g_free (_tmp6), NULL);
		g_spawn_async_with_pipes (working_dir, pars, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &self->priv->_child_pid, NULL, &stdo, &stde, &inner_error);
		if (inner_error != NULL) {
			cmd = (g_free (cmd), NULL);
			pars = (_vala_array_free (pars, pars_length1, (GDestroyNotify) g_free), NULL);
			start_message = (g_free (start_message), NULL);
			if (inner_error->domain == G_SPAWN_ERROR) {
				goto __catch11_g_spawn_error;
			}
			goto __finally11;
		}
		if (self->priv->_child_pid != ((GPid) 0)) {
			self->priv->_child_watch_id = g_child_watch_add (self->priv->_child_pid, _vtg_project_builder_on_child_watch_gchild_watch_func, self);
			vtg_build_log_view_initialize (self->priv->_build_view, NULL);
			if (self->priv->last_exit_code == 0) {
				gboolean _tmp7;
				self->priv->is_bottom_pane_visible = (g_object_get ((GtkWidget*) gedit_window_get_bottom_panel (vtg_plugin_instance_get_window (self->priv->_plugin_instance)), "visible", &_tmp7, NULL), _tmp7);
			}
			vtg_output_view_start_watch (log, self->priv->_child_watch_id, stdo, stde, -1);
			vtg_output_view_activate (log);
			g_signal_emit_by_name (self, "build-start");
		} else {
			vtg_output_view_log_message (log, "error compiling file\n");
		}
		return (_tmp8 = TRUE, cmd = (g_free (cmd), NULL), pars = (_vala_array_free (pars, pars_length1, (GDestroyNotify) g_free), NULL), start_message = (g_free (start_message), NULL), working_dir = (g_free (working_dir), NULL), (log == NULL) ? NULL : (log = (g_object_unref (log), NULL)), _tmp8);
	}
	goto __finally11;
	__catch11_g_spawn_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			char* msg;
			gboolean _tmp9;
			msg = g_strdup_printf ("error spawning compiler process: %s", err->message);
			g_warning (msg);
			vtg_output_view_log_message (log, msg);
			return (_tmp9 = FALSE, (err == NULL) ? NULL : (err = (g_error_free (err), NULL)), msg = (g_free (msg), NULL), working_dir = (g_free (working_dir), NULL), (log == NULL) ? NULL : (log = (g_object_unref (log), NULL)), _tmp9);
		}
	}
	__finally11:
	if (inner_error != NULL) {
		working_dir = (g_free (working_dir), NULL);
		(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return FALSE;
	}
	working_dir = (g_free (working_dir), NULL);
	(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
}


gboolean vtg_project_builder_build (VtgProjectBuilder* self, VtgProjectManager* project_manager, const char* params) {
	GError * inner_error;
	VbfProject* _tmp1;
	VbfProject* project;
	const char* _tmp2;
	char* working_dir;
	gint stdo;
	gint stde;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (project_manager != NULL, FALSE);
	inner_error = NULL;
	if (self->priv->_child_watch_id != 0) {
		return FALSE;
	}
	_tmp1 = NULL;
	project = (_tmp1 = vtg_project_manager_get_project (project_manager), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	_tmp2 = NULL;
	working_dir = (_tmp2 = project->id, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2));
	stdo = 0;
	stde = 0;
	{
		VtgOutputView* _tmp3;
		VtgOutputView* log;
		char* start_message;
		char* _tmp5;
		char* _tmp4;
		char* cmd;
		gint pars_size;
		gint pars_length1;
		char** pars;
		char* _tmp8;
		gboolean _tmp10;
		_tmp3 = NULL;
		log = (_tmp3 = vtg_plugin_instance_get_output_view (self->priv->_plugin_instance), (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3));
		vtg_output_view_clean_output (log);
		start_message = g_strdup_printf (_ ("Start building project: %s\n"), project->name);
		vtg_output_view_log_message (log, start_message);
		_tmp5 = NULL;
		_tmp4 = NULL;
		vtg_output_view_log_message (log, _tmp5 = g_strdup_printf ("%s\n\n", _tmp4 = g_strnfill ((gulong) (string_get_length (start_message) - 1), '-')));
		_tmp5 = (g_free (_tmp5), NULL);
		_tmp4 = (g_free (_tmp4), NULL);
		cmd = NULL;
		if (params != NULL) {
			char* _tmp6;
			_tmp6 = NULL;
			cmd = (_tmp6 = g_strdup_printf ("%s %s", VTG_PROJECT_BUILDER_MAKE, params), cmd = (g_free (cmd), NULL), _tmp6);
		} else {
			char* _tmp7;
			_tmp7 = NULL;
			cmd = (_tmp7 = g_strdup (VTG_PROJECT_BUILDER_MAKE), cmd = (g_free (cmd), NULL), _tmp7);
		}
		pars = (pars_length1 = 0, NULL);
		g_shell_parse_argv (cmd, &pars_length1, &pars, &inner_error);
		if (inner_error != NULL) {
			(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
			start_message = (g_free (start_message), NULL);
			cmd = (g_free (cmd), NULL);
			pars = (_vala_array_free (pars, pars_length1, (GDestroyNotify) g_free), NULL);
			if (inner_error->domain == G_SPAWN_ERROR) {
				goto __catch12_g_spawn_error;
			}
			goto __finally12;
		}
		_tmp8 = NULL;
		vtg_output_view_log_message (log, _tmp8 = g_strdup_printf ("%s\n", cmd));
		_tmp8 = (g_free (_tmp8), NULL);
		g_spawn_async_with_pipes (working_dir, pars, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &self->priv->_child_pid, NULL, &stdo, &stde, &inner_error);
		if (inner_error != NULL) {
			(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
			start_message = (g_free (start_message), NULL);
			cmd = (g_free (cmd), NULL);
			pars = (_vala_array_free (pars, pars_length1, (GDestroyNotify) g_free), NULL);
			if (inner_error->domain == G_SPAWN_ERROR) {
				goto __catch12_g_spawn_error;
			}
			goto __finally12;
		}
		if (self->priv->_child_pid != ((GPid) 0)) {
			self->priv->_child_watch_id = g_child_watch_add (self->priv->_child_pid, _vtg_project_builder_on_child_watch_gchild_watch_func, self);
			vtg_build_log_view_initialize (self->priv->_build_view, project_manager);
			if (self->priv->last_exit_code == 0) {
				gboolean _tmp9;
				self->priv->is_bottom_pane_visible = (g_object_get ((GtkWidget*) gedit_window_get_bottom_panel (vtg_plugin_instance_get_window (self->priv->_plugin_instance)), "visible", &_tmp9, NULL), _tmp9);
			}
			vtg_output_view_start_watch (log, self->priv->_child_watch_id, stdo, stde, -1);
			vtg_output_view_activate (log);
			g_signal_emit_by_name (self, "build-start");
		} else {
			vtg_output_view_log_message (log, "error spawning 'make' process\n");
		}
		return (_tmp10 = TRUE, (log == NULL) ? NULL : (log = (g_object_unref (log), NULL)), start_message = (g_free (start_message), NULL), cmd = (g_free (cmd), NULL), pars = (_vala_array_free (pars, pars_length1, (GDestroyNotify) g_free), NULL), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), working_dir = (g_free (working_dir), NULL), _tmp10);
	}
	goto __finally12;
	__catch12_g_spawn_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			gboolean _tmp11;
			g_warning ("vtgprojectbuilder.vala:153: Error spawning build process: %s", err->message);
			return (_tmp11 = FALSE, (err == NULL) ? NULL : (err = (g_error_free (err), NULL)), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), working_dir = (g_free (working_dir), NULL), _tmp11);
		}
	}
	__finally12:
	if (inner_error != NULL) {
		(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
		working_dir = (g_free (working_dir), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return FALSE;
	}
	(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
	working_dir = (g_free (working_dir), NULL);
}


gboolean vtg_project_builder_configure (VtgProjectBuilder* self, VtgProjectManager* project_manager, const char* params) {
	GError * inner_error;
	VbfProject* _tmp1;
	VbfProject* project;
	const char* _tmp2;
	char* working_dir;
	gint stdo;
	gint stde;
	char* configure_command;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (project_manager != NULL, FALSE);
	inner_error = NULL;
	if (self->priv->_child_watch_id != 0) {
		return FALSE;
	}
	_tmp1 = NULL;
	project = (_tmp1 = vtg_project_manager_get_project (project_manager), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	_tmp2 = NULL;
	working_dir = (_tmp2 = project->id, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2));
	stdo = 0;
	stde = 0;
	configure_command = NULL;
	{
		char** _tmp3;
		char** item_collection;
		int item_collection_length1;
		int item_it;
		_tmp3 = NULL;
		item_collection = (_tmp3 = g_new0 (char*, 2 + 1), _tmp3[0] = g_strdup ("./configure"), _tmp3[1] = g_strdup ("./autogen.sh"), _tmp3);
		item_collection_length1 = 2;
		for (item_it = 0; item_it < 2; item_it = item_it + 1) {
			const char* _tmp6;
			char* item;
			_tmp6 = NULL;
			item = (_tmp6 = item_collection[item_it], (_tmp6 == NULL) ? NULL : g_strdup (_tmp6));
			{
				char* file;
				file = g_build_filename (working_dir, item, NULL);
				if (g_file_test (file, G_FILE_TEST_EXISTS)) {
					char* _tmp5;
					const char* _tmp4;
					_tmp5 = NULL;
					_tmp4 = NULL;
					configure_command = (_tmp5 = (_tmp4 = item, (_tmp4 == NULL) ? NULL : g_strdup (_tmp4)), configure_command = (g_free (configure_command), NULL), _tmp5);
					item = (g_free (item), NULL);
					file = (g_free (file), NULL);
					break;
				}
				item = (g_free (item), NULL);
				file = (g_free (file), NULL);
			}
		}
		item_collection = (_vala_array_free (item_collection, item_collection_length1, (GDestroyNotify) g_free), NULL);
	}
	if (configure_command == NULL) {
		gboolean _tmp7;
		return (_tmp7 = FALSE, (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), working_dir = (g_free (working_dir), NULL), configure_command = (g_free (configure_command), NULL), _tmp7);
	}
	{
		VtgOutputView* _tmp8;
		VtgOutputView* log;
		char* start_message;
		char* _tmp10;
		char* _tmp9;
		char* cmd;
		gint pars_size;
		gint pars_length1;
		char** pars;
		char* _tmp14;
		gboolean _tmp17;
		_tmp8 = NULL;
		log = (_tmp8 = vtg_plugin_instance_get_output_view (self->priv->_plugin_instance), (_tmp8 == NULL) ? NULL : g_object_ref (_tmp8));
		vtg_output_view_clean_output (log);
		start_message = g_strdup_printf (_ ("Start configure project: %s\n"), project->name);
		vtg_output_view_log_message (log, start_message);
		_tmp10 = NULL;
		_tmp9 = NULL;
		vtg_output_view_log_message (log, _tmp10 = g_strdup_printf ("%s\n\n", _tmp9 = g_strnfill ((gulong) (string_get_length (start_message) - 1), '-')));
		_tmp10 = (g_free (_tmp10), NULL);
		_tmp9 = (g_free (_tmp9), NULL);
		cmd = NULL;
		if (params != NULL) {
			char* _tmp11;
			_tmp11 = NULL;
			cmd = (_tmp11 = g_strdup_printf ("%s %s", configure_command, params), cmd = (g_free (cmd), NULL), _tmp11);
		} else {
			char* _tmp13;
			const char* _tmp12;
			_tmp13 = NULL;
			_tmp12 = NULL;
			cmd = (_tmp13 = (_tmp12 = configure_command, (_tmp12 == NULL) ? NULL : g_strdup (_tmp12)), cmd = (g_free (cmd), NULL), _tmp13);
		}
		pars = (pars_length1 = 0, NULL);
		g_shell_parse_argv (cmd, &pars_length1, &pars, &inner_error);
		if (inner_error != NULL) {
			(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
			start_message = (g_free (start_message), NULL);
			cmd = (g_free (cmd), NULL);
			pars = (_vala_array_free (pars, pars_length1, (GDestroyNotify) g_free), NULL);
			if (inner_error->domain == G_SPAWN_ERROR) {
				goto __catch13_g_spawn_error;
			}
			goto __finally13;
		}
		_tmp14 = NULL;
		vtg_output_view_log_message (log, _tmp14 = g_strdup_printf ("%s\n", cmd));
		_tmp14 = (g_free (_tmp14), NULL);
		g_spawn_async_with_pipes (working_dir, pars, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &self->priv->_child_pid, NULL, &stdo, &stde, &inner_error);
		if (inner_error != NULL) {
			(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
			start_message = (g_free (start_message), NULL);
			cmd = (g_free (cmd), NULL);
			pars = (_vala_array_free (pars, pars_length1, (GDestroyNotify) g_free), NULL);
			if (inner_error->domain == G_SPAWN_ERROR) {
				goto __catch13_g_spawn_error;
			}
			goto __finally13;
		}
		if (self->priv->_child_pid != ((GPid) 0)) {
			self->priv->_child_watch_id = g_child_watch_add (self->priv->_child_pid, _vtg_project_builder_on_child_watch_gchild_watch_func, self);
			vtg_build_log_view_initialize (self->priv->_build_view, project_manager);
			if (self->priv->last_exit_code == 0) {
				gboolean _tmp15;
				self->priv->is_bottom_pane_visible = (g_object_get ((GtkWidget*) gedit_window_get_bottom_panel (vtg_plugin_instance_get_window (self->priv->_plugin_instance)), "visible", &_tmp15, NULL), _tmp15);
			}
			vtg_output_view_start_watch (log, self->priv->_child_watch_id, stdo, stde, -1);
			vtg_output_view_activate (log);
			g_signal_emit_by_name (self, "build-start");
		} else {
			char* _tmp16;
			_tmp16 = NULL;
			vtg_output_view_log_message (log, _tmp16 = g_strdup_printf (_ ("error spawning '%s' process\n"), configure_command));
			_tmp16 = (g_free (_tmp16), NULL);
		}
		return (_tmp17 = TRUE, (log == NULL) ? NULL : (log = (g_object_unref (log), NULL)), start_message = (g_free (start_message), NULL), cmd = (g_free (cmd), NULL), pars = (_vala_array_free (pars, pars_length1, (GDestroyNotify) g_free), NULL), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), working_dir = (g_free (working_dir), NULL), configure_command = (g_free (configure_command), NULL), _tmp17);
	}
	goto __finally13;
	__catch13_g_spawn_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			gboolean _tmp18;
			g_warning ("vtgprojectbuilder.vala:207: Error spawning build process: %s", err->message);
			return (_tmp18 = FALSE, (err == NULL) ? NULL : (err = (g_error_free (err), NULL)), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), working_dir = (g_free (working_dir), NULL), configure_command = (g_free (configure_command), NULL), _tmp18);
		}
	}
	__finally13:
	if (inner_error != NULL) {
		(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
		working_dir = (g_free (working_dir), NULL);
		configure_command = (g_free (configure_command), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return FALSE;
	}
	(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
	working_dir = (g_free (working_dir), NULL);
	configure_command = (g_free (configure_command), NULL);
}


gboolean vtg_project_builder_clean (VtgProjectBuilder* self, VtgProjectManager* project_manager, gboolean vala_stamp) {
	GError * inner_error;
	VbfProject* _tmp1;
	VbfProject* project;
	const char* _tmp2;
	char* working_dir;
	gint stdo;
	gint stde;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (project_manager != NULL, FALSE);
	inner_error = NULL;
	if (self->priv->_child_watch_id != 0) {
		return FALSE;
	}
	_tmp1 = NULL;
	project = (_tmp1 = vtg_project_manager_get_project (project_manager), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	_tmp2 = NULL;
	working_dir = (_tmp2 = project->working_dir, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2));
	stdo = 0;
	stde = 0;
	{
		VtgOutputView* _tmp3;
		VtgOutputView* log;
		char* start_message;
		char* _tmp5;
		char* _tmp4;
		char* _tmp11;
		gint _tmp13_length1;
		char** _tmp13;
		char** _tmp12;
		gboolean _tmp15;
		_tmp3 = NULL;
		log = (_tmp3 = vtg_plugin_instance_get_output_view (self->priv->_plugin_instance), (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3));
		vtg_output_view_clean_output (log);
		start_message = g_strdup_printf (_ ("Start cleaning project: %s\n"), project->name);
		vtg_output_view_log_message (log, start_message);
		_tmp5 = NULL;
		_tmp4 = NULL;
		vtg_output_view_log_message (log, _tmp5 = g_strdup_printf ("%s\n\n", _tmp4 = g_strnfill ((gulong) (string_get_length (start_message) - 1), '-')));
		_tmp5 = (g_free (_tmp5), NULL);
		_tmp4 = (g_free (_tmp4), NULL);
		if (vala_stamp) {
			char* _tmp6;
			char* command;
			char* _tmp7;
			gboolean _tmp8;
			_tmp6 = NULL;
			vtg_output_view_log_message (log, _tmp6 = g_strdup_printf (_ ("cleaning 'stamp' files for project: %s\n"), project->name));
			_tmp6 = (g_free (_tmp6), NULL);
			command = g_strdup_printf ("find %s -name *.stamp -delete", working_dir);
			_tmp7 = NULL;
			vtg_output_view_log_message (log, _tmp7 = g_strdup_printf ("%s\n\n", command));
			_tmp7 = (g_free (_tmp7), NULL);
			_tmp8 = g_spawn_command_line_sync (command, NULL, NULL, NULL, &inner_error);
			if (inner_error != NULL) {
				command = (g_free (command), NULL);
				(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
				start_message = (g_free (start_message), NULL);
				if (inner_error->domain == G_SPAWN_ERROR) {
					goto __catch14_g_spawn_error;
				}
				goto __finally14;
			}
			if (!_tmp8) {
				char* _tmp9;
				gboolean _tmp10;
				_tmp9 = NULL;
				vtg_output_view_log_message (log, _tmp9 = g_strdup_printf (_ ("error cleaning 'stamp' files for project: %s\n"), project->name));
				_tmp9 = (g_free (_tmp9), NULL);
				return (_tmp10 = FALSE, command = (g_free (command), NULL), (log == NULL) ? NULL : (log = (g_object_unref (log), NULL)), start_message = (g_free (start_message), NULL), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), working_dir = (g_free (working_dir), NULL), _tmp10);
			}
			command = (g_free (command), NULL);
		}
		_tmp11 = NULL;
		vtg_output_view_log_message (log, _tmp11 = g_strdup_printf ("%s %s\n", VTG_PROJECT_BUILDER_MAKE, "clean"));
		_tmp11 = (g_free (_tmp11), NULL);
		_tmp13 = NULL;
		_tmp12 = NULL;
		g_spawn_async_with_pipes (working_dir, (_tmp13 = (_tmp12 = g_new0 (char*, 2 + 1), _tmp12[0] = g_strdup (VTG_PROJECT_BUILDER_MAKE), _tmp12[1] = g_strdup ("clean"), _tmp12), _tmp13_length1 = 2, _tmp13), NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &self->priv->_child_pid, NULL, &stdo, &stde, &inner_error);
		if (inner_error != NULL) {
			(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
			start_message = (g_free (start_message), NULL);
			if (inner_error->domain == G_SPAWN_ERROR) {
				goto __catch14_g_spawn_error;
			}
			goto __finally14;
		}
		_tmp13 = (_vala_array_free (_tmp13, _tmp13_length1, (GDestroyNotify) g_free), NULL);
		if (self->priv->_child_pid != ((GPid) 0)) {
			self->priv->_child_watch_id = g_child_watch_add (self->priv->_child_pid, _vtg_project_builder_on_child_watch_gchild_watch_func, self);
			vtg_build_log_view_initialize (self->priv->_build_view, project_manager);
			if (self->priv->last_exit_code == 0) {
				gboolean _tmp14;
				self->priv->is_bottom_pane_visible = (g_object_get ((GtkWidget*) gedit_window_get_bottom_panel (vtg_plugin_instance_get_window (self->priv->_plugin_instance)), "visible", &_tmp14, NULL), _tmp14);
			}
			vtg_output_view_start_watch (log, self->priv->_child_watch_id, stdo, stde, -1);
			vtg_output_view_activate (log);
			g_signal_emit_by_name (self, "build-start");
		} else {
			vtg_output_view_log_message (log, "error spawning 'make clean' process\n");
		}
		return (_tmp15 = TRUE, (log == NULL) ? NULL : (log = (g_object_unref (log), NULL)), start_message = (g_free (start_message), NULL), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), working_dir = (g_free (working_dir), NULL), _tmp15);
	}
	goto __finally14;
	__catch14_g_spawn_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			gboolean _tmp16;
			g_warning ("vtgprojectbuilder.vala:252: Error spawning clean command: %s", err->message);
			return (_tmp16 = FALSE, (err == NULL) ? NULL : (err = (g_error_free (err), NULL)), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), working_dir = (g_free (working_dir), NULL), _tmp16);
		}
	}
	__finally14:
	if (inner_error != NULL) {
		(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
		working_dir = (g_free (working_dir), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return FALSE;
	}
	(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
	working_dir = (g_free (working_dir), NULL);
}


void vtg_project_builder_next_error (VtgProjectBuilder* self) {
	g_return_if_fail (self != NULL);
	vtg_build_log_view_next_error (self->priv->_build_view);
}


void vtg_project_builder_previous_error (VtgProjectBuilder* self) {
	g_return_if_fail (self != NULL);
	vtg_build_log_view_previous_error (self->priv->_build_view);
}


static void vtg_project_builder_on_child_watch (VtgProjectBuilder* self, GPid pid, gint status) {
	VtgOutputView* _tmp0;
	VtgOutputView* log;
	char* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	log = (_tmp0 = vtg_plugin_instance_get_output_view (self->priv->_plugin_instance), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	vtg_output_view_stop_watch (log, self->priv->_child_watch_id);
	self->priv->last_exit_code = WEXITSTATUS (status);
	_tmp1 = NULL;
	vtg_output_view_log_message (log, _tmp1 = g_strdup_printf (_ ("\ncompilation end with exit status %d\n"), self->priv->last_exit_code));
	_tmp1 = (g_free (_tmp1), NULL);
	vtg_build_log_view_activate (self->priv->_build_view);
	self->priv->_child_watch_id = (guint) 0;
	g_signal_emit_by_name (self, "build-exit", self->priv->last_exit_code);
	g_spawn_close_pid (pid);
	if (self->priv->last_exit_code == 0) {
		if (!self->priv->is_bottom_pane_visible) {
			gtk_widget_hide ((GtkWidget*) gedit_window_get_bottom_panel (vtg_plugin_instance_get_window (self->priv->_plugin_instance)));
		}
	} else {
		gdk_beep ();
	}
	self->priv->_child_pid = (GPid) 0;
	(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
}


void vtg_project_builder_stop_build (VtgProjectBuilder* self) {
	g_return_if_fail (self != NULL);
	if (((gint) self->priv->_child_pid) != 0) {
		if (kill ((gint) self->priv->_child_pid, 9) != 0) {
			g_warning ("vtgprojectbuilder.vala:293: stop build error: kill failed");
		} else {
			GMainContext* _tmp0;
			GMainContext* ctx;
			_tmp0 = NULL;
			ctx = (_tmp0 = g_main_context_default (), (_tmp0 == NULL) ? NULL : g_main_context_ref (_tmp0));
			while (TRUE) {
				gboolean _tmp1;
				_tmp1 = FALSE;
				if (self->priv->_child_watch_id != 0) {
					_tmp1 = g_main_context_pending (ctx);
				} else {
					_tmp1 = FALSE;
				}
				if (!_tmp1) {
					break;
				}
				g_main_context_iteration (ctx, FALSE);
			}
			(ctx == NULL) ? NULL : (ctx = (g_main_context_unref (ctx), NULL));
		}
	}
}


VtgPluginInstance* vtg_project_builder_get_plugin_instance (VtgProjectBuilder* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_plugin_instance;
}


static void vtg_project_builder_set_plugin_instance (VtgProjectBuilder* self, VtgPluginInstance* value) {
	VtgPluginInstance* _tmp2;
	VtgPluginInstance* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_plugin_instance = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_plugin_instance == NULL) ? NULL : (self->priv->_plugin_instance = (g_object_unref (self->priv->_plugin_instance), NULL)), _tmp2);
	g_object_notify ((GObject *) self, "plugin-instance");
}


VtgBuildLogView* vtg_project_builder_get_error_pane (VtgProjectBuilder* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_build_view;
}


gboolean vtg_project_builder_get_is_building (VtgProjectBuilder* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_child_watch_id != 0;
}


static GObject * vtg_project_builder_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgProjectBuilderClass * klass;
	GObjectClass * parent_class;
	VtgProjectBuilder * self;
	klass = VTG_PROJECT_BUILDER_CLASS (g_type_class_peek (VTG_TYPE_PROJECT_BUILDER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_PROJECT_BUILDER (obj);
	{
		VtgBuildLogView* _tmp0;
		gboolean _tmp1;
		_tmp0 = NULL;
		self->priv->_build_view = (_tmp0 = vtg_build_log_view_new (self->priv->_plugin_instance), (self->priv->_build_view == NULL) ? NULL : (self->priv->_build_view = (g_object_unref (self->priv->_build_view), NULL)), _tmp0);
		self->priv->is_bottom_pane_visible = (g_object_get ((GtkWidget*) gedit_window_get_bottom_panel (vtg_plugin_instance_get_window (self->priv->_plugin_instance)), "visible", &_tmp1, NULL), _tmp1);
	}
	return obj;
}


static void vtg_project_builder_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgProjectBuilder * self;
	gpointer boxed;
	self = VTG_PROJECT_BUILDER (object);
	switch (property_id) {
		case VTG_PROJECT_BUILDER_PLUGIN_INSTANCE:
		g_value_set_object (value, vtg_project_builder_get_plugin_instance (self));
		break;
		case VTG_PROJECT_BUILDER_ERROR_PANE:
		g_value_set_object (value, vtg_project_builder_get_error_pane (self));
		break;
		case VTG_PROJECT_BUILDER_IS_BUILDING:
		g_value_set_boolean (value, vtg_project_builder_get_is_building (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_builder_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgProjectBuilder * self;
	self = VTG_PROJECT_BUILDER (object);
	switch (property_id) {
		case VTG_PROJECT_BUILDER_PLUGIN_INSTANCE:
		vtg_project_builder_set_plugin_instance (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_builder_class_init (VtgProjectBuilderClass * klass) {
	vtg_project_builder_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgProjectBuilderPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_project_builder_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_project_builder_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_project_builder_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_project_builder_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_BUILDER_PLUGIN_INSTANCE, g_param_spec_object ("plugin-instance", "plugin-instance", "plugin-instance", VTG_TYPE_PLUGIN_INSTANCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_BUILDER_ERROR_PANE, g_param_spec_object ("error-pane", "error-pane", "error-pane", VTG_TYPE_BUILD_LOG_VIEW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_BUILDER_IS_BUILDING, g_param_spec_boolean ("is-building", "is-building", "is-building", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_signal_new ("build_start", VTG_TYPE_PROJECT_BUILDER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("build_exit", VTG_TYPE_PROJECT_BUILDER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
}


static void vtg_project_builder_instance_init (VtgProjectBuilder * self) {
	self->priv = VTG_PROJECT_BUILDER_GET_PRIVATE (self);
	self->priv->_build_view = NULL;
	self->priv->_child_watch_id = (guint) 0;
	self->priv->last_exit_code = 0;
}


static void vtg_project_builder_finalize (GObject* obj) {
	VtgProjectBuilder * self;
	self = VTG_PROJECT_BUILDER (obj);
	(self->priv->_plugin_instance == NULL) ? NULL : (self->priv->_plugin_instance = (g_object_unref (self->priv->_plugin_instance), NULL));
	(self->priv->_build_view == NULL) ? NULL : (self->priv->_build_view = (g_object_unref (self->priv->_build_view), NULL));
	G_OBJECT_CLASS (vtg_project_builder_parent_class)->finalize (obj);
}


static GType vtg_project_builder_type_id = 0;
GType vtg_project_builder_get_type (void) {
	return vtg_project_builder_type_id;
}


GType vtg_project_builder_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectBuilderClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_builder_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectBuilder), 0, (GInstanceInitFunc) vtg_project_builder_instance_init, NULL };
	vtg_project_builder_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectBuilder", &g_define_type_info, 0);
	return vtg_project_builder_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}




